@@ -17,16 +17,12 @@
 """ Base tests cases and test utilities """
 from __future__ import with_statement
 
-import dbus
-from dbus.mainloop.glib import DBusGMainLoop
 import contextlib
 import logging
 import os
 import shutil
 import itertools
 
-from ubuntu_sso import DBUS_IFACE_CRED_NAME
-from ubuntu_sso.main import SSOCredentials
 from ubuntuone.syncdaemon import (
     config,
     action_queue,
@@ -36,18 +32,14 @@ from ubuntuone.syncdaemon import (
     volume_manager,
     main,
     local_rescan,
-    logger,
-)
-from ubuntuone.syncdaemon.dbus_interface import (
-    DBusInterface,
-    DBusExposedObject,
-    NM_STATE_CONNECTED,
-    NM_STATE_DISCONNECTED,
+    tritcask,
 )
+from ubuntuone.syncdaemon import logger
+logger.init()
 from twisted.internet import defer
-from twisted.python import failure
 from twisted.trial.unittest import TestCase as TwistedTestCase
 from zope.interface import implements
+from zope.interface.verify import verifyObject
 
 
 FAKED_CREDENTIALS = {'consumer_key': 'faked_consumer_key',
@@ -92,20 +84,19 @@ class FakeHashQueue(object):
                      crc32='', size=0, stat=os.stat(path))
 
 
-
 class FakeMark(object):
     """A fake Mark Shuttleworth..."""
     def stop(self):
         """...that only knows how to stop"""
 
 
-class FakeDBusInterface(object):
+class FakeExternalInterface(object):
     """A fake DBusInterface..."""
 
     def shutdown(self, with_restart=False):
         """...that only knows how to go away"""
 
-    def _request_token(self):
+    def _request_token(self, *args, **kwargs):
         """Return a token which is a fixed set of credentials."""
         return FAKED_CREDENTIALS
 
@@ -118,17 +109,40 @@ class FakeActionQueue(object):
     def __init__(self, eq, *args, **kwargs):
         """ Creates the instance """
         self.eq = self.event_queue = eq
-        self.uploading = {}
-        self.downloading = {}
         self.uuid_map = action_queue.DeferredMap()
-        self.content_queue = action_queue.ContentQueue('CONTENT_QUEUE', self)
-        self.meta_queue = action_queue.RequestQueue('META_QUEUE', self)
+        self.queue = action_queue.RequestQueue(self)
 
         # throttling attributes
         self.readLimit = None
         self.writeLimit = None
         self.throttling_enabled = False
 
+    def __setattr__(self, attr, value):
+        """Custom __setattr__ that check the interface.
+
+        After setting a callable attribute, verify the interface.
+        """
+        r = super(FakeActionQueue, self).__setattr__(attr, value)
+        if callable(value):
+            # check that AQ implements IActionQueue.
+            verifyObject(interfaces.IActionQueue, self)
+        return r
+
+    # IContentQueue
+    def cancel_download(self, share_id, node_id):
+        """Stub implementation."""
+
+    def cancel_upload(self, share_id, node_id):
+        """Stub implementation."""
+
+    def download(self, share_id, node_id, server_hash, path, fileobj_factory):
+        """Stub implementation."""
+
+    def upload(self, share_id, node_id, previous_hash, hash, crc32,
+               size, path, fileobj_factory, tempfile_factory=None):
+        """Stub implementation."""
+
+    # IMetaQueue
     def connect(self, host=None, port=None, user_ssl=False):
         """Just send connect!."""
         self.eq.push('SYS_CONNECTION_MADE')
@@ -143,20 +157,23 @@ class FakeActionQueue(object):
 
     def answer_share(self, share_id, answer):
         """Send the event."""
-        self.eq.push('AQ_ANSWER_SHARE_OK', share_id, answer)
-
-    def delete_share(self, share_id):
-        """Stub."""
+        self.eq.push('AQ_ANSWER_SHARE_OK', share_id=share_id, answer=answer)
 
     def disconnect(self, *a, **k):
         """Stub implementation."""
 
     cancel_download = cancel_upload = download = upload = make_dir = disconnect
     make_file = move = unlink = list_shares = disconnect
-    list_volumes = create_share = inquire_free_space =  disconnect
+    list_volumes = create_share = create_udf = inquire_free_space =  disconnect
     inquire_account_info = delete_volume = change_public_access = disconnect
-    query_volumes = get_delta = rescan_from_scratch = disconnect
-    node_is_with_queued_move = disconnect
+    query_volumes = get_delta = rescan_from_scratch = delete_share = disconnect
+    node_is_with_queued_move = cleanup = get_public_files = disconnect
+
+
+class FakeStatusListener(object):
+    """A fake StatusListener."""
+
+    show_all_notifications = True
 
 
 class FakeMain(main.Main):
@@ -176,9 +193,10 @@ class FakeMain(main.Main):
         self.shares_dir = shares_dir
         self.partials_dir = partials_dir
         self.shares_dir_link = os.path.join(self.root_dir, 'Shared With Me')
+        self.db = tritcask.Tritcask(os.path.join(self.data_dir, 'tritcask'))
         self.vm = volume_manager.VolumeManager(self)
-        self.fs = fs_manager.FileSystemManager(self.data_dir,
-                                               self.partials_dir, self.vm)
+        self.fs = fs_manager.FileSystemManager(
+            self.data_dir, self.partials_dir, self.vm, self.db)
         self.event_q = event_queue.EventQueue(self.fs)
         self.fs.register_eq(self.event_q)
         self.action_q = self._fake_AQ_class(self.event_q, self,
@@ -190,9 +208,10 @@ class FakeMain(main.Main):
         self.vm.init_root()
         self.hash_q = FakeHashQueue(self.event_q)
         self.mark = FakeMark()
-        self.dbus_iface = FakeDBusInterface()
+        self.external = FakeExternalInterface()
         self.lr = local_rescan.LocalRescan(self.vm, self.fs,
                                            self.event_q, self.action_q)
+        self.status_listener = FakeStatusListener()
 
     def _connect_aq(self, _):
         """Connect the fake action queue."""
@@ -315,11 +334,11 @@ class BaseTwistedTestCase(TwistedTestCase):
         return TwistedTestCase.tearDown(self)
 
 
-class DBusTwistedTestCase(BaseTwistedTestCase):
-    """ Test the DBus event handling """
+class FakeMainTestCase(BaseTwistedTestCase):
+    """A testcase that starts up a Main instance."""
 
     def setUp(self):
-        """ Setup the infrastructure fo the test (dbus service). """
+        """Setup the infrastructure for the test."""
         BaseTwistedTestCase.setUp(self)
         self.log = logging.getLogger("ubuntuone.SyncDaemon.TEST")
         self.log.info("starting test %s.%s", self.__class__.__name__,
@@ -331,43 +350,16 @@ class DBusTwistedTestCase(BaseTwistedTestCase):
         self.shares_dir = self.mktemp('shares_dir')
         self.main = FakeMain(self.root_dir, self.shares_dir,
                              self.data_dir, self.partials_dir)
-        self.fs_manager = self.main.fs
+        self.vm = self.main.vm
+        self.fs = self.main.fs
         self.event_q = self.main.event_q
         self.action_q = self.main.action_q
-        self.loop = DBusGMainLoop(set_as_default=True)
-        self.bus = dbus.bus.BusConnection(mainloop=self.loop)
-        self.nm = FakeNetworkManager(self.bus)
-        # monkeypatch busName.__del__ to avoid errors on gc
-        # we take care of releasing the name in shutdown
-        dbus.service.BusName.__del__ = lambda _: None
-        self.dbus_iface = DBusInterface(self.bus, self.main,
-                                        system_bus=self.bus)
-        # token is a fixed set of credentials
-        self.dbus_iface._request_token = lambda: FAKED_CREDENTIALS
-
-        self.main.dbus_iface = self.dbus_iface
-        self.busName = self.dbus_iface.busName
-        self.bus.set_exit_on_disconnect(False)
-        self.dbus_iface.connect()
         self.event_q.push('SYS_INIT_DONE')
-        self.signal_receivers = set()
-        self.action_q.content_queue.set_change_notification_cb(
-            self.dbus_iface.status.emit_content_queue_changed)
 
     def tearDown(self):
-        """ Cleanup the test. """
-        d = self.cleanup_signal_receivers(self.signal_receivers)
-        d.addBoth(self._tearDown)
-        d.addBoth(lambda _: BaseTwistedTestCase.tearDown(self))
-        return d
-
-    def _tearDown(self, *args):
-        """ shutdown """
+        """Shutdown this testcase."""
+        BaseTwistedTestCase.tearDown(self)
         self.main.shutdown()
-        self.nm.shutdown()
-        self.dbus_iface.bus.flush()
-        self.bus.flush()
-        self.bus.close()
         self.rmtree(self.shares_dir)
         self.rmtree(self.root_dir)
         self.rmtree(self.data_dir)
@@ -375,128 +367,6 @@ class DBusTwistedTestCase(BaseTwistedTestCase):
         self.log.info("finished test %s.%s", self.__class__.__name__,
                       self._testMethodName)
 
-    def error_handler(self, error):
-        """ default error handler for DBus calls. """
-        if isinstance(error, failure.Failure):
-            self.fail(error.getErrorMessage())
-        else:
-            self.fail(str(error))
-
-    def cleanup_signal_receivers(self, signal_receivers):
-        """ cleanup self.signal_receivers and returns a deferred """
-        deferreds = []
-        for match in signal_receivers:
-            d = defer.Deferred()
-            def callback(*args):
-                """ callback that accepts *args. """
-                if not d.called:
-                    d.callback(args)
-            self.bus.call_async(dbus.bus.BUS_DAEMON_NAME,
-                                dbus.bus.BUS_DAEMON_PATH,
-                                dbus.bus.BUS_DAEMON_IFACE, 'RemoveMatch', 's',
-                                (str(match),), callback, self.error_handler)
-            deferreds.append(d)
-        if deferreds:
-            return defer.DeferredList(deferreds)
-        else:
-            return defer.succeed(True)
-
-
-class MementoHandler(logging.Handler):
-    """ A handler class which store logging records in a list """
-
-    def __init__(self, *args, **kwargs):
-        """ Create the instance, and add a records attribute. """
-        logging.Handler.__init__(self, *args, **kwargs)
-        self.records = []
-        self.debug = False
-
-    def emit(self, record):
-        """ Just add the record to self.records. """
-        self.records.append(record)
-
-    def check(self, level, *msgs):
-        """Verifies that the msgs are logged in the specified level."""
-        for rec in self.records:
-            if rec.levelno == level and all(m in rec.message for m in msgs):
-                return True
-        if self.debug:
-            recorded =  [(logging.getLevelName(r.levelno), r.message)
-                         for r in self.records]
-            print "Memento messages:", recorded
-        return False
-
-    def check_debug(self, *msgs):
-        """Shortcut for checking in DEBUG."""
-        return self.check(logging.DEBUG, *msgs)
-
-    def check_info(self, *msgs):
-        """Shortcut for checking in INFO."""
-        return self.check(logging.INFO, *msgs)
-
-    def check_warning(self, *msgs):
-        """Shortcut for checking in WARNING."""
-        return self.check(logging.WARNING, *msgs)
-
-    def check_error(self, *msgs):
-        """Shortcut for checking in ERROR."""
-        return self.check(logging.ERROR, *msgs)
-
-    def check_note(self, *msgs):
-        """Shortcut for checking in NOTE."""
-        return self.check(logger.NOTE, *msgs)
-
-
-class FakeNetworkManager(DBusExposedObject):
-    """ A fake NetworkManager that only emits StatusChanged signal. """
-
-    State = 3
-
-    def __init__(self, bus):
-        """ Creates the instance. """
-        self.path = '/org/freedesktop/NetworkManager'
-        self.bus = bus
-        self.bus.request_name('org.freedesktop.NetworkManager',
-                              flags=dbus.bus.NAME_FLAG_REPLACE_EXISTING | \
-                              dbus.bus.NAME_FLAG_DO_NOT_QUEUE | \
-                              dbus.bus.NAME_FLAG_ALLOW_REPLACEMENT)
-        self.busName = dbus.service.BusName('org.freedesktop.NetworkManager',
-                                        bus=self.bus)
-        DBusExposedObject.__init__(self, bus_name=self.busName,
-                                   path=self.path)
-
-    def shutdown(self):
-        """ Shutdown the fake NetworkManager """
-        self.busName.get_bus().release_name(self.busName.get_name())
-        self.remove_from_connection()
-
-    @dbus.service.signal('org.freedesktop.NetworkManager', signature='i')
-    # pylint: disable-msg=C0103
-    def StateChanged(self, state):
-        """ Fire DBus signal StatusChanged. """
-        pass
-
-    def emit_connected(self):
-        """ Emits the signal StateCganged(3). """
-        self.StateChanged(NM_STATE_CONNECTED)
-
-    def emit_disconnected(self):
-        """ Emits the signal StateCganged(4). """
-        self.StateChanged(NM_STATE_DISCONNECTED)
-
-    @dbus.service.method(dbus.PROPERTIES_IFACE,
-                         in_signature='ss', out_signature='v',
-                         async_callbacks=('reply_handler', 'error_handler'))
-    # pylint: disable-msg=C0103
-    def Get(self, interface, propname, reply_handler=None, error_handler=None):
-        """
-        Fake dbus's Get method to get at the State property
-        """
-        try:
-            reply_handler(getattr(self, propname, None))
-        except Exception, e: # pylint: disable-msg=W0703
-            error_handler(e)
-
 
 class FakeVolumeManager(object):
     """ A volume manager that only knows one share, the root"""
@@ -530,7 +400,10 @@ class FakeVolumeManager(object):
         try:
             return self.shares[id]
         except KeyError:
-            return self.udfs[id]
+            try:
+                return self.udfs[id]
+            except KeyError:
+                raise volume_manager.VolumeDoesNotExist(id)
 
     def get_volumes(self, all_volumes=False):
         """Simple get_volumes for FakeVolumeManager."""
@@ -549,62 +422,6 @@ class FakeVolumeManager(object):
         """Request the deletion of a volume."""
 
 
-# OAuth stubs
-class FakeLoginProcessor(object):
-    """Stub login processor."""
-
-    def __init__(self, dbus_object):
-        """Initialize the login processor."""
-        self.next_login_cb = None
-
-    def login(self, realm, consumer_key, error_handler=None,
-              reply_handler=None, do_login=True):
-        """Stub, call self.next_login_cb or send NewCredentials if
-        self.next_login_cb isn't defined.
-        """
-        self.realm = str(realm)
-        self.consumer_key = str(consumer_key)
-        if self.next_login_cb:
-            cb = self.next_login_cb[0]
-            args = self.next_login_cb[1]
-            self.next_login_cb = None
-            return cb(*args)
-        else:
-            self.dbus_object.NewCredentials(realm, consumer_key)
-
-    def clear_token(self, realm, consumer_key):
-        """Stub, do nothing"""
-        pass
-
-    def next_login_with(self, callback, args=tuple()):
-        """shortcircuit the next call to login and call the specified callback.
-        callback is usually one of: self.got_token, self.got_no_token,
-        self.got_denial or self.got_error.
-        """
-        self.next_login_cb = (callback, args)
-
-
-class FakeLogin(SSOCredentials):
-    """Stub Object which listens for D-Bus OAuth requests"""
-
-    def __init__(self, bus):
-        """Initiate the object."""
-        self.bus = bus
-        self.busName = dbus.service.BusName(DBUS_IFACE_CRED_NAME, bus=self.bus)
-        # bypass the parent class __init__ as it has the path hardcoded
-        # and we can't use '/' as the path, as we are already using it
-        # for syncdaemon. pylint: disable-msg=W0233,W0231
-        dbus.service.Object.__init__(self, object_path="/oauthdesktop",
-                                     bus_name=self.busName)
-        self.processor = FakeLoginProcessor(self)
-        self.currently_authing = False
-
-    def shutdown(self):
-        """Shutdown and remove any trace from the bus"""
-        self.busName.get_bus().release_name(self.busName.get_name())
-        self.remove_from_connection()
-
-
 class FakeLogger(object):
     """Helper logging class."""
     def __init__(self):
@@ -635,9 +452,9 @@ class Listener(object):
     def __init__(self):
         self.events = []
 
-    def handle_default(self, event_name, *args, **kwargs):
+    def handle_default(self, event_name, **kwargs):
         """Keep record of every event."""
-        self.events.append((event_name, args, kwargs))
+        self.events.append((event_name, kwargs))
 
 
 class DummyClass(object):
@@ -646,4 +463,3 @@ class DummyClass(object):
     def __getattr__(self, name):
         """Any attribute is a no-op."""
         return lambda *args, **kwargs: None
-
