@@ -58,34 +58,29 @@ test_daemon (void)
 static void
 test_config (void)
 {
-	gint orig_download, orig_upload, download, upload;
-	SyncdaemonConfigInterface *config;
 	gboolean orig_enabled, enabled;
 
-	config = (SyncdaemonConfigInterface *) syncdaemon_daemon_get_config_interface (the_daemon);
-	g_assert (SYNCDAEMON_IS_CONFIG_INTERFACE (config));
-
 	/* Test set/get_throttling_enabled */
-	orig_enabled = syncdaemon_config_interface_get_bandwidth_throttling (config);
-	syncdaemon_config_interface_set_bandwidth_throttling (config, !orig_enabled);
-	enabled = syncdaemon_config_interface_get_bandwidth_throttling (config);
+	orig_enabled = syncdaemon_daemon_get_bandwidth_throttling (the_daemon);
+	syncdaemon_daemon_set_bandwidth_throttling (the_daemon, !orig_enabled);
+	enabled = syncdaemon_daemon_get_bandwidth_throttling (the_daemon);
 	g_assert (orig_enabled != enabled);
 
-	syncdaemon_config_interface_set_bandwidth_throttling (config, orig_enabled);
-	enabled = syncdaemon_config_interface_get_bandwidth_throttling (config);
+	syncdaemon_daemon_set_bandwidth_throttling (the_daemon, orig_enabled);
+	enabled = syncdaemon_daemon_get_bandwidth_throttling (the_daemon);
 	g_assert (orig_enabled == enabled);
 
 	/* FIXME: we don't test get/set_files_sync since disabling it means the
 	   syncdaemon quits and doesn't start anymore */
 
 	/* Test set/get_udf_autosubscribe */
-	orig_enabled = syncdaemon_config_interface_get_udf_autosubscribe (config);
-	syncdaemon_config_interface_set_udf_autosubscribe (config, !orig_enabled);
-	enabled = syncdaemon_config_interface_get_udf_autosubscribe (config);
+	orig_enabled = syncdaemon_daemon_get_udf_autosubscribe (the_daemon);
+	syncdaemon_daemon_set_udf_autosubscribe (the_daemon, !orig_enabled);
+	enabled = syncdaemon_daemon_get_udf_autosubscribe (the_daemon);
 	g_assert (orig_enabled != enabled);
 
-	syncdaemon_config_interface_set_udf_autosubscribe (config, orig_enabled);
-	enabled = syncdaemon_config_interface_get_udf_autosubscribe (config);
+	syncdaemon_daemon_set_udf_autosubscribe (the_daemon, orig_enabled);
+	enabled = syncdaemon_daemon_get_udf_autosubscribe (the_daemon);
 	g_assert (orig_enabled == enabled);
 }
 
@@ -193,13 +188,9 @@ static void
 test_folders (void)
 {
 	GSList *folders_list, *l;
-	SyncdaemonFoldersInterface *folders;
-
-	folders = (SyncdaemonFoldersInterface *) syncdaemon_daemon_get_folders_interface (the_daemon);
-	g_assert (SYNCDAEMON_IS_FOLDERS_INTERFACE (folders));
 
 	/* Get list of UDFs */
-	folders_list = syncdaemon_folders_interface_get_folders (folders);
+	folders_list = syncdaemon_daemon_get_folders (the_daemon);
 	g_assert (g_slist_length (folders_list) >= 1);
 	
 	for (l = folders_list; l != NULL; l = l->next) {
@@ -336,13 +327,9 @@ static void
 test_shares (void)
 {
 	GSList *list;
-	SyncdaemonSharesInterface *shares;
-
-	shares = (SyncdaemonSharesInterface *) syncdaemon_daemon_get_shares_interface (the_daemon);
-	g_assert (SYNCDAEMON_IS_SHARES_INTERFACE (shares));
 
 	/* Get shared folders */
-	list = syncdaemon_shares_interface_get_shared (shares);
+	list = syncdaemon_daemon_get_shared_folders (the_daemon);
 	while (list != NULL) {
 		SyncdaemonShareInfo *share_info = list->data;
 
@@ -353,7 +340,7 @@ test_shares (void)
 	}
 
 	/* Get shares */
-	list = syncdaemon_shares_interface_get_shares (shares);
+	list = syncdaemon_daemon_get_shares (the_daemon);
 	while (list != NULL) {
 		SyncdaemonShareInfo *share_info = list->data;
 
@@ -368,13 +355,9 @@ static void
 test_status (void)
 {
 	GSList *list;
-	SyncdaemonStatusInterface *status;
-
-	status = (SyncdaemonStatusInterface *) syncdaemon_daemon_get_status_interface (the_daemon);
-	g_assert (SYNCDAEMON_IS_STATUS_INTERFACE (status));
 
 	/* Test getting current downloads */
-	list = syncdaemon_status_interface_get_current_downloads (status);
+	list = syncdaemon_daemon_get_current_downloads (the_daemon);
 	while (list != NULL) {
 		SyncdaemonTransferInfo *tinfo = (SyncdaemonTransferInfo *) list->data;
 
@@ -383,7 +366,7 @@ test_status (void)
 	}
 
 	/* Test getting current uploads */
-	list = syncdaemon_status_interface_get_current_uploads (status);
+	list = syncdaemon_daemon_get_current_uploads (the_daemon);
 	while (list != NULL) {
 		SyncdaemonTransferInfo *tinfo = (SyncdaemonTransferInfo *) list->data;
 
@@ -467,7 +450,7 @@ test_transfer_info (void)
 static void
 daemon_connected_cb (SyncdaemonDaemon *daemon, gpointer user_data)
 {
-	g_debug ("Daemon is now connected, and root_dir = ", syncdaemon_daemon_get_root_dir (the_daemon));
+	g_debug ("Daemon is now connected, and root_dir = %s", syncdaemon_daemon_get_root_dir (the_daemon));
 	syncdaemon_daemon_disconnect (the_daemon);
 }
 
