@@ -176,12 +176,12 @@ ubuntuone_is_location_bar_enabled (void)
 	show_location_value = gconf_client_get (conf_client, EXPANDER_SHOWN_KEY, NULL);
 	g_object_unref (conf_client);
 
-	if (show_location_value == NULL)
+	if (show_location_value == NULL) {
 		show_location = TRUE;
-	else
+	} else {
 		show_location = gconf_value_get_bool (show_location_value);
-
-	gconf_value_free (show_location_value);
+		gconf_value_free (show_location_value);
+	}
 
 	return show_location;
 }
