@@ -74,6 +74,7 @@ from ubuntuone.syncdaemon.volume_manager import Share, Shared, UDF
 from tests.syncdaemon.test_action_queue import (
     ConnectedBaseTestCase,
     FakeSemaphore,
+    TestingProtocol,
 )
 
 VOLUME = uuid.UUID('12345678-1234-1234-1234-123456789abc')
@@ -606,7 +607,8 @@ class ZeitgeistRemoteFileSyncTestCase(ConnectedBaseTestCase):
         cmd.handle_success(request)
 
         # create a request and fill it with succesful information
-        request = client.PutContent(self.action_queue.client, self.share_id,
+        aq_client = TestingProtocol()
+        request = client.PutContent(aq_client, self.share_id,
                                     'node', 'prvhash', 'newhash', 'crc32',
                                     'size', 'deflated', 'fd')
         request.new_generation = 13
@@ -695,7 +697,8 @@ class ZeitgeistRemoteFileSyncTestCase(ConnectedBaseTestCase):
         self.fsm.set_node_id(path, "a_node_id")
 
         # create a request and fill it with succesful information
-        request = client.PutContent(self.action_queue.client, self.share_id,
+        aq_client = TestingProtocol()
+        request = client.PutContent(aq_client, self.share_id,
                                     'node', 'prvhash', 'newhash', 'crc32',
                                     'size', 'deflated', 'fd')
         request.new_generation = 13
