@@ -1202,7 +1202,7 @@ class StatusAggregatorTestCase(TestCase):
         self.assertEqual(self.aggregator.done_counter, 0)
         self.assertEqual(len(self.aggregator.files_uploading), 0)
         self.assertEqual(len(self.aggregator.files_downloading), 0)
-        self.assertIs(self.aggregator.queue_done_timer, None)
+        self.assertIdentical(self.aggregator.queue_done_timer, None)
 
     def assertMiscCommandQueued(self, fc):
         """Assert that some command was queued."""
@@ -1386,7 +1386,7 @@ class StatusAggregatorTestCase(TestCase):
         self.aggregator.clock.advance(self.aggregator.finished_delay / 2)
         fc2 = FakeCommand()
         self.status_frontend.download_started(fc2)
-        self.assertIs(self.aggregator.queue_done_timer, None)
+        self.assertIdentical(self.aggregator.queue_done_timer, None)
         self.aggregator.clock.advance(self.aggregator.finished_delay)
         self.status_frontend.download_finished(fc2)
 
@@ -1400,7 +1400,7 @@ class StatusAggregatorTestCase(TestCase):
         self.aggregator.clock.advance(self.aggregator.finished_delay / 2)
         fc2 = FakeCommand()
         self.status_frontend.upload_started(fc2)
-        self.assertIs(self.aggregator.queue_done_timer, None)
+        self.assertIdentical(self.aggregator.queue_done_timer, None)
         self.aggregator.clock.advance(self.aggregator.finished_delay)
         self.status_frontend.upload_finished(fc2)
 
