@@ -428,6 +428,45 @@ class TestConfigBasic(BaseTwistedTestCase):
         branch_config = os.path.join(rootdir, "data", config.CONFIG_FILE)
         self.assertIn(branch_config, config_files)
 
+    def test_get_simult_transfers(self):
+        """Get simult transfers."""
+        conf_file = os.path.join(self.test_root, 'test_load_config.conf')
+        with open(conf_file, 'w') as fh:
+            fh.write('[__main__]\n')
+            fh.write('simult_transfers = 12345\n')
+        conf = config._Config(conf_file)
+        self.assertEqual(conf.get_simult_transfers(), 12345)
+
+    def test_set_simult_transfers(self):
+        """Set simult transfers."""
+        conf_file = os.path.join(self.test_root, 'test_load_config.conf')
+        with open(conf_file, 'w') as fh:
+            fh.write('[__main__]\n')
+            fh.write('simult_transfers = 12345\n')
+        conf = config._Config(conf_file)
+        conf.set_simult_transfers(666)
+        self.assertEqual(conf.get_simult_transfers(), 666)
+
+    def test_get_max_payload_size(self):
+        """Get the maximum payload size."""
+        conf_file = os.path.join(self.test_root, 'test_load_config.conf')
+        with open(conf_file, 'w') as fh:
+            fh.write('[__main__]\n')
+            fh.write('max_payload_size = 12345\n')
+        conf = config._Config(conf_file)
+        self.assertEqual(conf.get_max_payload_size(), 12345)
+
+    def test_set_max_payload_size(self):
+        """Set the maximum payload size."""
+        conf_file = os.path.join(self.test_root, 'test_load_config.conf')
+        with open(conf_file, 'w') as fh:
+            fh.write('[__main__]\n')
+            fh.write('max_payload_size = 12345\n')
+        conf = config._Config(conf_file)
+        conf.set_max_payload_size(666)
+        self.assertEqual(conf.get_max_payload_size(), 666)
+
+
 class ConfigglueParsersTests(BaseTwistedTestCase):
     """Tests for our custom configglue parsers"""
 
@@ -603,3 +642,14 @@ class SyncDaemonConfigParserTests(BaseTwistedTestCase):
         self.cp.parse_all()
         self.assertIn(r'\A\.~lock\..*#\Z',
                       self.cp.get('__main__', 'ignore').value)
+
+    def test_simult_transfers(self):
+        """Test default configuration for simultaneous transfers."""
+        self.cp.parse_all()
+        self.assertEqual(self.cp.get('__main__', 'simult_transfers').value, 10)
+
+    def test_max_payload_size(self):
+        """Test default configuration for maximum payload size."""
+        self.cp.parse_all()
+        configured = self.cp.get('__main__', 'max_payload_size').value
+        self.assertEqual(configured, 1400)
