@@ -3,7 +3,7 @@
 #
 # Author: Guillermo Gonzalez <guillermo.gonzalez@canonical.com>
 #
-# Copyright 2009 Canonical Ltd.
+# Copyright 2009-2011 Canonical Ltd.
 #
 # This program is free software: you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 3, as published
@@ -25,9 +25,11 @@ import unittest
 from ubuntuone.syncdaemon import (
     event_queue,
     filesystem_manager,
+    tritcask,
 )
 from contrib.testing import testcase
 from twisted.internet import defer
+from ubuntuone.devtools.handlers import MementoHandler
 
 
 class BaseEQTestCase(testcase.BaseTwistedTestCase):
@@ -41,9 +43,10 @@ class BaseEQTestCase(testcase.BaseTwistedTestCase):
         self.root_dir = self.mktemp('root_dir')
         self.home_dir = self.mktemp('home_dir')
         self.vm = testcase.FakeVolumeManager(self.root_dir)
+        self.db = tritcask.Tritcask(self.mktemp('tritcask'))
         self.fs = filesystem_manager.FileSystemManager(self.fsmdir,
                                                        self.partials_dir,
-                                                       self.vm)
+                                                       self.vm, self.db)
         self.fs.create(path=self.root_dir,
                        share_id='', is_dir=True)
         self.fs.set_by_path(path=self.root_dir,
@@ -52,15 +55,15 @@ class BaseEQTestCase(testcase.BaseTwistedTestCase):
         self.fs.register_eq(self.eq)
 
         # add a Memento handler to the logger
-        self.log_handler = testcase.MementoHandler()
+        self.log_handler = MementoHandler()
         self.log_handler.setLevel(logging.DEBUG)
         self.eq.log.addHandler(self.log_handler)
 
     def tearDown(self):
         """Clean up the tests."""
-        for listener in self.eq._listeners:
-            self.eq.unsubscribe(listener)
+        self.eq.listener_map = {}
         self.eq.shutdown()
+        self.db.shutdown()
         self.rmtree(self.tmpdir)
         testcase.BaseTwistedTestCase.tearDown(self)
 
@@ -68,50 +71,121 @@ class BaseEQTestCase(testcase.BaseTwistedTestCase):
 class SubscriptionTests(BaseEQTestCase):
     """Test to subscribe and unsubscribe to the EQ."""
 
-    def test_subscription(self):
-        """Test that subscription works."""
-        # pylint: disable-msg=W0212
-        # simple one
-        o = object()
-        self.eq.subscribe(o)
-        self.assertTrue(o in self.eq._listeners)
+    def test_subscription_simple(self):
+        """Subscribe creating the listener map."""
+        class Listener(object):
+            """Listener."""
 
-        # not duplicates
-        o = object()
-        self.eq.subscribe(o)
-        self.eq.subscribe(o)
-        self.assertTrue(self.eq._listeners.count(o) == 1)
+            def handle_FS_FILE_CREATE(self, path):
+                """Handle FS_FILE_CREATE."""
+
+            def handle_default(self, *a):
+                """Handle all."""
+
+        # subscribe
+        listener = Listener()
+        self.eq.subscribe(listener)
+
+        # for FS_FILE_CREATE, it should have picked up the method
+        self.assertEqual(self.eq.listener_map['FS_FILE_CREATE'][listener],
+                         listener.handle_FS_FILE_CREATE)
+
+        # for any other event, it should have picked up the method
+        self.assertEqual(self.eq.listener_map['FS_DIR_DELETE'][listener].func,
+                         listener.handle_default)
+
+    def test_subscription_nodefault(self):
+        """Don't subscribe if there's no default."""
+        class Listener(object):
+            """Listener."""
+
+            def handle_FS_FILE_CREATE(self, path):
+                """Handle FS_FILE_CREATE."""
+
+        # subscribe
+        listener = Listener()
+        self.eq.subscribe(listener)
+
+        # for FS_FILE_CREATE, it should have picked up the method
+        self.assertEqual(self.eq.listener_map['FS_FILE_CREATE'][listener],
+                         listener.handle_FS_FILE_CREATE)
+
+        # for any other event, nothing
+        self.assertFalse('FS_DIR_DELETE' in self.eq.listener_map)
+
+    def test_subscription_two_listeners(self):
+        """Subscribe several listeners."""
+        class Listener1(object):
+            """Listener 1."""
+
+            def handle_FS_FILE_CREATE(self, path):
+                """Handle FS_FILE_CREATE."""
+
+            def handle_FS_DIR_CREATE(self, path):
+                """Handle FS_DIR_CREATE."""
+
+        class Listener2(object):
+            """Listener 2."""
+
+            def handle_FS_FILE_CREATE(self, path):
+                """Handle FS_FILE_CREATE."""
+
+            def handle_FS_DIR_DELETE(self, path):
+                """Handle FS_DIR_CREATE."""
+
+        # subscribe
+        listener1 = Listener1()
+        self.eq.subscribe(listener1)
+        listener2 = Listener2()
+        self.eq.subscribe(listener2)
+
+        # for FS_FILE_CREATE, both listeners should be there
+        lmap = self.eq.listener_map['FS_FILE_CREATE']
+        self.assertEqual(lmap[listener1], listener1.handle_FS_FILE_CREATE)
+        self.assertEqual(lmap[listener2], listener2.handle_FS_FILE_CREATE)
+
+        # for FS_DIR_CREATE and FS_DIR_DELETE, each one accordingly
+        self.assertEqual(self.eq.listener_map['FS_DIR_CREATE'][listener1],
+                         listener1.handle_FS_DIR_CREATE)
+        self.assertEqual(self.eq.listener_map['FS_DIR_DELETE'][listener2],
+                         listener2.handle_FS_DIR_DELETE)
 
     def test_unsubscription(self):
         """Test that unsubscription works."""
-        # pylint: disable-msg=W0212
-        # simple one
-        o = object()
-        self.eq.subscribe(o)
-        self.eq.unsubscribe(o)
-        self.assertFalse(o in self.eq._listeners)
+        class Listener1(object):
+            """Listener 1."""
 
-        # duplicate ok (two suscs)
-        o = object()
-        self.eq.subscribe(o)
-        self.eq.subscribe(o)
-        self.eq.unsubscribe(o)
-        self.assertFalse(o in self.eq._listeners)
+            def handle_FS_FILE_CREATE(self, path):
+                """Handle FS_FILE_CREATE."""
 
-        # duplicate error (two unsuscs)
-        o = object()
-        self.eq.subscribe(o)
-        self.eq.subscribe(o)
-        self.eq.unsubscribe(o)
-        self.assertRaises(ValueError, self.eq.unsubscribe, o)
+            def handle_FS_DIR_CREATE(self, path):
+                """Handle FS_DIR_CREATE."""
 
-        # indecisive
-        o = object()
-        self.eq.subscribe(o)
-        self.eq.unsubscribe(o)
-        self.eq.subscribe(o)
-        self.eq.unsubscribe(o)
-        self.assertFalse(o in self.eq._listeners)
+        class Listener2(object):
+            """Listener 2."""
+
+            def handle_FS_FILE_CREATE(self, path):
+                """Handle FS_FILE_CREATE."""
+
+            def handle_FS_DIR_DELETE(self, path):
+                """Handle FS_DIR_CREATE."""
+
+        # subscribe two listeners
+        listener1 = Listener1()
+        self.eq.subscribe(listener1)
+        listener2 = Listener2()
+        self.eq.subscribe(listener2)
+
+        # unsubscribe listener 1 only
+        self.eq.unsubscribe(listener1)
+
+        # check
+        self.assertFalse(listener1 in self.eq.listener_map['FS_FILE_CREATE'])
+        self.assertFalse('FS_DIR_CREATE' in self.eq.listener_map)
+        self.assertEqual(self.eq.listener_map['FS_FILE_CREATE'][listener2],
+                         listener2.handle_FS_FILE_CREATE)
+        self.assertEqual(self.eq.listener_map['FS_DIR_DELETE'][listener2],
+                         listener2.handle_FS_DIR_DELETE)
 
 
 class PushTests(BaseEQTestCase):
@@ -122,22 +196,13 @@ class PushTests(BaseEQTestCase):
         # not even an event
         self.assertRaises(TypeError, self.eq.push)
 
-        # bad event
-        self.assertRaises(event_queue.InvalidEventError,
-                          self.eq.push, "no-such-event")
-
-        # not enough or incorrect args for that event
-        self.assertRaises(TypeError, self.eq.push, "FS_FILE_MOVE")
-        self.assertRaises(TypeError, self.eq.push, "FS_FILE_MOVE", 1, 2, 3)
+        # incorrect args, only kwargs supported
+        self.assertRaises(TypeError, self.eq.push, "FS_FILE_MOVE", 1)
         self.assertRaises(TypeError,
-                      self.eq.push, "FS_FILE_MOVE", 1, 2, path_to=3)
-        self.assertRaises(TypeError,
-                      self.eq.push, "FS_FILE_MOVE", 1, path_from=2, path_to=3)
+                      self.eq.push, "FS_FILE_MOVE", 1, path_to=2)
 
-        # all ok... args, kwargs, and mixed
-        self.eq.push("FS_FILE_MOVE", 1, 2)
+        # ok: just kwargs
         self.eq.push("FS_FILE_MOVE", path_from=1, path_to=2)
-        self.eq.push("FS_FILE_MOVE", 1, path_to=2)
 
     def test_events_kwargs(self):
         """Test that all events are defined correctly with tuples or lists.
@@ -156,20 +221,20 @@ class PushTests(BaseEQTestCase):
         class Create(object):
             def __init__(self):
                 self.a = None
-            def handle_FS_FILE_CREATE(self, a):
-                self.a = a
+            def handle_FS_FILE_CREATE(self, path):
+                self.a = path
 
         # it get passed!
         c = Create()
         self.eq.subscribe(c)
-        self.eq.push("FS_FILE_CREATE", 1)
+        self.eq.push("FS_FILE_CREATE", path=1)
         self.assertEqual(c.a, 1)
         self.eq.unsubscribe(c)
 
         # don't get what don't listen
         c = Create()
         self.eq.subscribe(c)
-        self.eq.push("FS_FILE_DELETE", 1)
+        self.eq.push("FS_FILE_DELETE", path=1)
         self.assertEqual(c.a, None)
         self.eq.unsubscribe(c)
 
@@ -178,20 +243,16 @@ class PushTests(BaseEQTestCase):
 
         # helper class, pylint: disable-msg=C0111
         class Create(object):
-            def handle_FS_FILE_CREATE(self, a, b): # it should be only 'a' here
+            def handle_FS_FILE_CREATE(self, notpath): # it should be path here
                 pass
 
         # it get passed!
         c = Create()
         self.eq.subscribe(c)
 
-        # caught by EQ
-        self.assertRaises(TypeError, self.eq.push, "FS_FILE_CREATE")
-        self.assertRaises(TypeError, self.eq.push, "FS_FILE_CREATE", 1, 2)
-
-        # approved by EQ, but the listener has a wrong signature
+        # the listener has a wrong signature
         # this is logged as an error/exception
-        self.eq.push("FS_FILE_CREATE", 1)
+        self.eq.push("FS_FILE_CREATE", path=1)
         self.assertTrue(self.log_handler.check_error('FS_FILE_CREATE',
                                                      'Create object'))
 
@@ -199,13 +260,14 @@ class PushTests(BaseEQTestCase):
 
     def test_log_pushing_data(self):
         """Pushed event and info should be logged."""
-        self.eq.push("AQ_QUERY_ERROR", 'item', error='err')
+        self.eq.push("AQ_QUERY_ERROR", item='item', error='err')
         self.assertTrue(self.log_handler.check_debug(
-            "push_event: AQ_QUERY_ERROR, args:('item',), kw:{'error': 'err'}"))
+                        "push_event: AQ_QUERY_ERROR, kwargs: "
+                        "{'item': 'item', 'error': 'err'}"))
 
     def test_log_delete_in_info(self):
         """Pushed any deletion event should be logged in info."""
-        self.eq.push("FS_DIR_DELETE", 'path')
+        self.eq.push("FS_DIR_DELETE", path='path')
         self.assertTrue(self.log_handler.check_info(
                         "push_event: FS_DIR_DELETE"))
 
@@ -213,7 +275,7 @@ class PushTests(BaseEQTestCase):
         """SYS_USER_CONNECT event info must not be logged."""
         self.eq.push("SYS_USER_CONNECT", access_token='foo')
         self.assertTrue(self.log_handler.check_debug(
-            "push_event: SYS_USER_CONNECT, args:*, kw:*"))
+            "push_event: SYS_USER_CONNECT, kwargs: *"))
 
 
 class PushTestsWithCallback(BaseEQTestCase):
@@ -226,12 +288,12 @@ class PushTestsWithCallback(BaseEQTestCase):
         d = defer.Deferred()
         # helper class, pylint: disable-msg=C0111
         class BadListener(object):
-            def handle_FS_FILE_CREATE(self, a, b): # it should be only 'a' here
+            def handle_FS_FILE_CREATE(self, notpath): # it should be path here
                 d.callback(False)
 
         class GoodListener(object):
-            def handle_FS_FILE_CREATE(self, a):
-                d.callback(a)
+            def handle_FS_FILE_CREATE(self, path):
+                d.callback(path)
 
         bl = BadListener()
         gl = GoodListener()
@@ -244,12 +306,8 @@ class PushTestsWithCallback(BaseEQTestCase):
             self.eq.unsubscribe(gl)
         self.addCleanup(cleanup)
 
-        # caught by EQ
-        self.assertRaises(TypeError, self.eq.push, "FS_FILE_CREATE")
-        self.assertRaises(TypeError, self.eq.push, "FS_FILE_CREATE", 1, 2)
-
-        # approved by EQ, but one listener has a wrong signature
-        self.eq.push("FS_FILE_CREATE", 1)
+        # one listener has a wrong signature
+        self.eq.push("FS_FILE_CREATE", path=1)
         def callback(result):
             """ asserts that GoodListener was called. """
             self.assertTrue(result)
@@ -263,8 +321,8 @@ class PushTestsWithCallback(BaseEQTestCase):
         d = defer.Deferred()
         # helper class, pylint: disable-msg=C0111
         class Listener(object):
-            def handle_default(self, *args, **kwargs):
-                d.callback(args)
+            def handle_default(self, event, **kwargs):
+                d.callback((event, kwargs))
 
         l = Listener()
         self.eq.subscribe(l)
@@ -275,18 +333,18 @@ class PushTestsWithCallback(BaseEQTestCase):
         self.addCleanup(cleanup)
 
         # push some event and expect it'll be handled by handle_default
-        self.eq.push("FS_FILE_CREATE", 1)
+        self.eq.push("FS_FILE_CREATE", path=1)
         def callback(result):
             """ asserts that GoodListener was called. """
             self.assertEquals(2, len(result))
             self.assertEquals('FS_FILE_CREATE', result[0])
-            self.assertEquals(1, result[1])
+            self.assertEquals({'path': 1}, result[1])
 
         d.addCallback(callback)
         return d
 
     def test_ordered_dispatch(self):
-        """ Check that the events are pushed to all listeners in order. """
+        """Check that the events are pushed to all listeners in order."""
         d = defer.Deferred()
         # helper class, pylint: disable-msg=C0111
         class Listener(object):
@@ -294,14 +352,14 @@ class PushTestsWithCallback(BaseEQTestCase):
                 self.eq = eq
                 self.events = []
 
-            def handle_FS_FILE_CREATE(self, a):
+            def handle_FS_FILE_CREATE(self, path):
                 self.events.append('FS_FILE_CREATE')
-                self.eq.push('FS_FILE_MOVE', a, 2)
+                self.eq.push('FS_FILE_MOVE', path_from=path, path_to=2)
 
-            def handle_FS_FILE_DELETE(self, a):
+            def handle_FS_FILE_DELETE(self, path):
                 self.events.append('FS_FILE_DELETE')
 
-            def handle_FS_FILE_MOVE(self, *args):
+            def handle_FS_FILE_MOVE(self, path_from, path_to):
                 self.events.append('FS_FILE_MOVE')
                 d.callback(True)
 
@@ -313,18 +371,12 @@ class PushTestsWithCallback(BaseEQTestCase):
             listeners.append(l)
             self.eq.subscribe(l)
 
-        def cleanup():
-            """ unsubscribe the listeners """
-            for l in listeners:
-                self.eq.unsubscribe(l)
-        self.addCleanup(cleanup)
-
         # push some events to unleash the event madness
-        self.eq.push("FS_FILE_CREATE", 1)
-        self.eq.push('FS_FILE_DELETE', 2)
+        self.eq.push("FS_FILE_CREATE", path=1)
+        self.eq.push('FS_FILE_DELETE', path=2)
 
         def callback(result):
-            """ asserts that Listener was called in the right order"""
+            """Assert that Listener was called in the right order."""
             listeners_events = [listener.events for listener in listeners]
             for l_events in listeners_events:
                 for other_l_events in listeners_events:
@@ -334,107 +386,6 @@ class PushTestsWithCallback(BaseEQTestCase):
         return d
 
 
-class MuteFilterTests(unittest.TestCase):
-    """Tests the MuteFilter class."""
-
-    def setUp(self):
-        self.mf = event_queue.MuteFilter()
-
-    def test_empty(self):
-        """Nothing there."""
-        self.assertFalse(self.mf._cnt)
-
-    def test_add_one(self):
-        """Adds one element."""
-        self.mf.add("foo")
-        self.assertEqual(self.mf._cnt, dict(foo=1))
-
-    def test_add_two_different(self):
-        """Adds two different elements."""
-        self.mf.add("foo")
-        self.mf.add("bar")
-        self.assertEqual(self.mf._cnt, dict(foo=1, bar=1))
-
-    def test_add_two_equal(self):
-        """Adds one element twice."""
-        self.mf.add("foo")
-        self.mf.add("foo")
-        self.assertEqual(self.mf._cnt, dict(foo=2))
-
-    def test_add_two_equal_and_third(self):
-        """Adds one element."""
-        self.mf.add("foo")
-        self.mf.add("bar")
-        self.mf.add("bar")
-        self.assertEqual(self.mf._cnt, dict(foo=1, bar=2))
-
-    def test_pop_simple(self):
-        """Pops one element."""
-        self.mf.add("foo")
-        self.assertFalse(self.mf.pop("bar"))
-        self.assertEqual(self.mf._cnt, dict(foo=1))
-        self.assertTrue(self.mf.pop("foo"))
-        self.assertFalse(self.mf._cnt)
-
-    def test_pop_complex(self):
-        """Pops several elements."""
-        # add several
-        self.mf.add("foo")
-        self.mf.add("bar")
-        self.mf.add("bar")
-        self.assertEqual(self.mf._cnt, dict(foo=1, bar=2))
-
-        # clean bar
-        self.assertTrue(self.mf.pop("bar"))
-        self.assertEqual(self.mf._cnt, dict(foo=1, bar=1))
-        self.assertTrue(self.mf.pop("bar"))
-        self.assertEqual(self.mf._cnt, dict(foo=1))
-        self.assertFalse(self.mf.pop("bar"))
-        self.assertEqual(self.mf._cnt, dict(foo=1))
-
-        # clean foo
-        self.assertTrue(self.mf.pop("foo"))
-        self.assertFalse(self.mf._cnt)
-        self.assertFalse(self.mf.pop("foo"))
-        self.assertFalse(self.mf._cnt)
-
-
-class IgnoreFileTests(unittest.TestCase):
-    """Tests the ignore files behaviour."""
-
-    def test_filter_none(self):
-        """Still works ok even if not receiving a regex to ignore."""
-        p = event_queue._GeneralINotifyProcessor(None)
-        self.assertFalse(p.is_ignored("froo.pyc"))
-
-    def test_filter_one(self):
-        """Filters stuff that matches (or not) this one regex."""
-        p = event_queue._GeneralINotifyProcessor(None, ['\A.*\\.pyc\Z'])
-        self.assertTrue(p.is_ignored("froo.pyc"))
-        self.assertFalse(p.is_ignored("froo.pyc.real"))
-        self.assertFalse(p.is_ignored("otherstuff"))
-
-    def test_filter_two_simple(self):
-        """Filters stuff that matches (or not) these simple regexes."""
-        p = event_queue._GeneralINotifyProcessor(None, ['\A.*foo\Z',
-                                                        '\A.*bar\Z'])
-        self.assertTrue(p.is_ignored("blah_foo"))
-        self.assertTrue(p.is_ignored("blah_bar"))
-        self.assertFalse(p.is_ignored("bar_xxx"))
-        self.assertFalse(p.is_ignored("--foo--"))
-        self.assertFalse(p.is_ignored("otherstuff"))
-
-    def test_filter_two_complex(self):
-        """Filters stuff that matches (or not) these complex regexes."""
-        p = event_queue._GeneralINotifyProcessor(None, ['\A.*foo\Z|\Afoo.*\Z',
-                                                        '\A.*bar\Z'])
-        self.assertTrue(p.is_ignored("blah_foo"))
-        self.assertTrue(p.is_ignored("blah_bar"))
-        self.assertTrue(p.is_ignored("foo_xxx"))
-        self.assertFalse(p.is_ignored("--foo--"))
-        self.assertFalse(p.is_ignored("otherstuff"))
-
-
 def test_suite():
     # pylint: disable-msg=C0111
     return unittest.TestLoader().loadTestsFromName(__name__)
