@@ -18,16 +18,16 @@
 
 from __future__ import with_statement
 
+import inspect
 import os
 import shutil
 import unittest
 
-from twisted.trial.unittest import TestCase as TwistedTestCase
 from twisted.internet import defer, reactor
 
 from contrib.testing import testcase
 from ubuntuone.syncdaemon import (filesystem_manager, event_queue,
-                                  events_nanny, hash_queue)
+                                  events_nanny, hash_queue, tritcask)
 
 TESTS_DIR = os.path.join(os.getcwd(), "tmp")
 
@@ -37,9 +37,9 @@ class EventListener(object):
     def __init__(self):
         self._events = []
 
-    def handle_default(self, event, *args):
+    def handle_default(self, event, **args):
         """Store the received event."""
-        self._events.append((event,) + args)
+        self._events.append((event,) + tuple(sorted(args.itervalues())))
 
     def events(self):
         """Clean and return events."""
@@ -47,6 +47,7 @@ class EventListener(object):
         self._events = []
         return tmp
 
+
 def hash_hack_run(self):
     """Function to hack the hash class to avoid it processing the requests."""
     while True:
@@ -58,12 +59,13 @@ def hash_hack_run(self):
         self.hashing = path
 
 
-class DownloadFinishedTests(TwistedTestCase):
+class DownloadFinishedTests(testcase.BaseTwistedTestCase):
     """Test the AQ Download Finished Nanny behaviour."""
     timeout = 2
 
     def setUp(self):
         """set up the test."""
+        testcase.BaseTwistedTestCase.setUp(self)
         try:
             os.mkdir(TESTS_DIR)
         except OSError:
@@ -79,8 +81,10 @@ class DownloadFinishedTests(TwistedTestCase):
 
         # create vm, fsm, eq, hq...
         vm = testcase.FakeVolumeManager(self.usrdir)
-        self.fsm = fsm = filesystem_manager.FileSystemManager(self.usrdir,
-                                                      self.partials_dir, vm)
+        db = tritcask.Tritcask(self.mktemp('tritcask'))
+        self.addCleanup(db.shutdown)
+        self.fsm = fsm = filesystem_manager.FileSystemManager(
+            self.usrdir, self.partials_dir, vm, db)
         self.eq = eq = event_queue.EventQueue(fsm)
         self.addCleanup(eq.shutdown)
         self.hq = hq = hash_queue.HashQueue(eq)
@@ -97,6 +101,7 @@ class DownloadFinishedTests(TwistedTestCase):
         """tear down the test."""
         hash_queue._Hasher.run = self._original_hash_run
         self.hq.shutdown()
+        testcase.BaseTwistedTestCase.tearDown(self)
 
     def insert_in_hq(self, path, node_id):
         """Inserts something in HQ and waits that thread."""
@@ -127,7 +132,8 @@ class DownloadFinishedTests(TwistedTestCase):
 
     def test_forward(self):
         """Forwards the event when file is not blocked."""
-        self.eq.push("AQ_DOWNLOAD_COMMIT", "", "nodeid", "s_hash")
+        self.eq.push("AQ_DOWNLOAD_COMMIT",
+                     share_id="", node_id="nodeid", server_hash="s_hash")
 
         def check(_):
             self.assertEqual(self.listener.events(), [
@@ -143,8 +149,9 @@ class DownloadFinishedTests(TwistedTestCase):
 
     def test_blocks_when_open(self):
         """Blocks the event if the file is opened."""
-        self.eq.push("FS_FILE_OPEN", self.tf)
-        self.eq.push("AQ_DOWNLOAD_COMMIT", "", "nodeid", "s_hash")
+        self.eq.push("FS_FILE_OPEN", path=self.tf)
+        self.eq.push("AQ_DOWNLOAD_COMMIT",
+                     share_id="", node_id="nodeid", server_hash="s_hash")
 
         def check(_):
             self.assertEqual(self.listener.events(), [
@@ -167,15 +174,17 @@ class DownloadFinishedTests(TwistedTestCase):
             self.assertTrue(self.tf in self.nanny._blocked)
 
         d = self.insert_in_hq(self.tf, "nodeid")
-        d.addCallback(lambda _: self.eq.push("AQ_DOWNLOAD_COMMIT", "",
-                                             "nodeid", "s_hash"))
+        d.addCallback(lambda _: self.eq.push("AQ_DOWNLOAD_COMMIT", share_id="",
+                                             node_id="nodeid",
+                                             server_hash="s_hash"))
         d.addCallback(check)
         return d
 
     def test_closenowrite(self):
         """A close_nowrite received, but no file was blocked."""
-        self.eq.push("FS_FILE_OPEN", self.tf)
-        self.eq.push("AQ_DOWNLOAD_COMMIT", "", "nodeid", "s_hash")
+        self.eq.push("FS_FILE_OPEN", path=self.tf)
+        self.eq.push("AQ_DOWNLOAD_COMMIT",
+                     share_id="", node_id="nodeid", server_hash="s_hash")
 
         def check1(_):
             self.assertEqual(self.listener.events(), [
@@ -192,15 +201,17 @@ class DownloadFinishedTests(TwistedTestCase):
 
         d = defer.Deferred()
         d.addCallback(check1)
-        d.addCallback(lambda _: self.eq.push("FS_FILE_CLOSE_WRITE", self.tf))
+        d.addCallback(lambda _: self.eq.push("FS_FILE_CLOSE_WRITE",
+                                             path=self.tf))
         d.addCallback(check2)
         d.callback(None)
         return d
 
     def test_blocks_closewrite(self):
         """Blocks the event and does NOT release it when close write."""
-        self.eq.push("FS_FILE_OPEN", self.tf)
-        self.eq.push("AQ_DOWNLOAD_COMMIT", "", "nodeid", "s_hash")
+        self.eq.push("FS_FILE_OPEN", path=self.tf)
+        self.eq.push("AQ_DOWNLOAD_COMMIT",
+                     share_id="", node_id="nodeid", server_hash="s_hash")
 
         def check1(_):
             self.assertEqual(self.listener.events(), [
@@ -219,15 +230,17 @@ class DownloadFinishedTests(TwistedTestCase):
 
         d = defer.Deferred()
         d.addCallback(check1)
-        d.addCallback(lambda _: self.eq.push("FS_FILE_CLOSE_WRITE", self.tf))
+        d.addCallback(lambda _: self.eq.push("FS_FILE_CLOSE_WRITE",
+                                             path=self.tf))
         d.addCallback(check2)
         d.callback(None)
         return d
 
     def test_blocks_release_close(self):
         """Blocks the event and releases it when close."""
-        self.eq.push("FS_FILE_OPEN", self.tf)
-        self.eq.push("AQ_DOWNLOAD_COMMIT", "", "nodeid", "s_hash")
+        self.eq.push("FS_FILE_OPEN", path=self.tf)
+        self.eq.push("AQ_DOWNLOAD_COMMIT",
+                     share_id="", node_id="nodeid", server_hash="s_hash")
 
         def check1(_):
             self.assertEqual(self.listener.events(), [
@@ -247,15 +260,17 @@ class DownloadFinishedTests(TwistedTestCase):
 
         d = defer.Deferred()
         d.addCallback(check1)
-        d.addCallback(lambda _: self.eq.push("FS_FILE_CLOSE_NOWRITE", self.tf))
+        d.addCallback(lambda _: self.eq.push("FS_FILE_CLOSE_NOWRITE",
+                                             path=self.tf))
         d.addCallback(check2)
         d.callback(None)
         return d
 
     def test_blocks_release_hash_doubleopen(self):
         """Blocks the event and releases it when hashed, double mixed open."""
-        self.eq.push("FS_FILE_OPEN", self.tf)
-        self.eq.push("AQ_DOWNLOAD_COMMIT", "", "nodeid", "s_hash")
+        self.eq.push("FS_FILE_OPEN", path=self.tf)
+        self.eq.push("AQ_DOWNLOAD_COMMIT",
+                     share_id="", node_id="nodeid", server_hash="s_hash")
 
         def check1(_):
             self.assertEqual(self.listener.events(), [
@@ -292,7 +307,7 @@ class DownloadFinishedTests(TwistedTestCase):
 
         def check5(_):
             self.assertEqual(self.listener.events(), [
-                        ("HQ_HASH_NEW", self.tf, "hash", "crc", "siz", "stt"),
+                        ("HQ_HASH_NEW", self.tf, "crc", "hash", "size", "stt"),
                         ("AQ_DOWNLOAD_FINISHED", "", "nodeid", "s_hash"),
             ])
             self.assertFalse(self.tf in self.nanny._blocked)
@@ -301,23 +316,27 @@ class DownloadFinishedTests(TwistedTestCase):
 
         d = defer.Deferred()
         d.addCallback(check1)
-        d.addCallback(lambda _: self.eq.push("FS_FILE_CLOSE_WRITE", self.tf))
+        d.addCallback(lambda _: self.eq.push("FS_FILE_CLOSE_WRITE",
+                                             path=self.tf))
         d.addCallback(check2)
-        d.addCallback(lambda _: self.eq.push("FS_FILE_OPEN", self.tf))
+        d.addCallback(lambda _: self.eq.push("FS_FILE_OPEN", path=self.tf))
         d.addCallback(check3)
-        d.addCallback(lambda _: self.eq.push("FS_FILE_CLOSE_NOWRITE", self.tf))
+        d.addCallback(lambda _: self.eq.push("FS_FILE_CLOSE_NOWRITE",
+                                             path=self.tf))
         d.addCallback(check4)
-        d.addCallback(lambda _: self.eq.push("HQ_HASH_NEW", self.tf, "hash",
-                                             "crc", "siz", "stt"))
+        d.addCallback(lambda _: self.eq.push("HQ_HASH_NEW", path=self.tf,
+                                             hash="hash", crc32="crc",
+                                             size="size", stat="stt"))
         d.addCallback(check5)
         d.callback(None)
         return d
 
     def test_blocks_release_close_doubleopen(self):
         """Blocks the event and releases it when close, double open."""
-        self.eq.push("FS_FILE_OPEN", self.tf)
-        self.eq.push("AQ_DOWNLOAD_COMMIT", "", "nodeid", "s_hash")
-        self.eq.push("FS_FILE_OPEN", self.tf)
+        self.eq.push("FS_FILE_OPEN", path=self.tf)
+        self.eq.push("AQ_DOWNLOAD_COMMIT",
+                     share_id="", node_id="nodeid", server_hash="s_hash")
+        self.eq.push("FS_FILE_OPEN", path=self.tf)
 
         def check1(_):
             self.assertEqual(self.listener.events(), [
@@ -345,17 +364,20 @@ class DownloadFinishedTests(TwistedTestCase):
 
         d = defer.Deferred()
         d.addCallback(check1)
-        d.addCallback(lambda _: self.eq.push("FS_FILE_CLOSE_NOWRITE", self.tf))
+        d.addCallback(lambda _: self.eq.push("FS_FILE_CLOSE_NOWRITE",
+                                             path=self.tf))
         d.addCallback(check2)
-        d.addCallback(lambda _: self.eq.push("FS_FILE_CLOSE_NOWRITE", self.tf))
+        d.addCallback(lambda _: self.eq.push("FS_FILE_CLOSE_NOWRITE",
+                                             path=self.tf))
         d.addCallback(check3)
         d.callback(None)
         return d
 
     def test_blocks_release_close_differentfiles(self):
         """Blocks the event and releases it when close, several files."""
-        self.eq.push("FS_FILE_OPEN", self.tf)
-        self.eq.push("AQ_DOWNLOAD_COMMIT", "", "nodeid", "s_hash")
+        self.eq.push("FS_FILE_OPEN", path=self.tf)
+        self.eq.push("AQ_DOWNLOAD_COMMIT",
+                     share_id="", node_id="nodeid", server_hash="s_hash")
 
         def check1(_):
             self.assertEqual(self.listener.events(), [
@@ -379,9 +401,11 @@ class DownloadFinishedTests(TwistedTestCase):
 
         d = defer.Deferred()
         d.addCallback(check1)
-        d.addCallback(lambda _: self.eq.push("FS_FILE_CLOSE_NOWRITE", "other"))
+        d.addCallback(lambda _: self.eq.push("FS_FILE_CLOSE_NOWRITE",
+                                             path="other"))
         d.addCallback(check2)
-        d.addCallback(lambda _: self.eq.push("FS_FILE_CLOSE_NOWRITE", self.tf))
+        d.addCallback(lambda _: self.eq.push("FS_FILE_CLOSE_NOWRITE",
+                                             path=self.tf))
         d.addCallback(check3)
         d.callback(None)
         return d
@@ -396,25 +420,28 @@ class DownloadFinishedTests(TwistedTestCase):
 
         def check2(_):
             self.assertEqual(self.listener.events(), [
-                        ("HQ_HASH_NEW", self.tf, "hash", "crc", "siz", "stt"),
+                        ("HQ_HASH_NEW", self.tf, "crc", "hash", "siz", "stt"),
                         ("AQ_DOWNLOAD_FINISHED", "", "nodeid", "s_hash"),
             ])
             self.assertFalse(self.tf in self.nanny._blocked)
 
         d = self.insert_in_hq(self.tf, "nodeid")
-        d.addCallback(lambda _: self.eq.push("AQ_DOWNLOAD_COMMIT", "",
-                                             "nodeid", "s_hash"))
+        d.addCallback(lambda _: self.eq.push("AQ_DOWNLOAD_COMMIT", share_id="",
+                                             node_id="nodeid",
+                                             server_hash="s_hash"))
         d.addCallback(check1)
         d.addCallback(lambda _: self.release_hq())
-        d.addCallback(lambda _: self.eq.push("HQ_HASH_NEW", self.tf, "hash",
-                                             "crc", "siz", "stt"))
+        d.addCallback(lambda _: self.eq.push("HQ_HASH_NEW", path=self.tf,
+                                             hash="hash", crc32="crc",
+                                             size="siz", stat="stt"))
         d.addCallback(check2)
         return d
 
     def test_blocks_closewrite_hashdone(self):
         """Knows that is hashing also because of CLOSE_WRITE."""
-        self.eq.push("FS_FILE_OPEN", self.tf)
-        self.eq.push("AQ_DOWNLOAD_COMMIT", "", "nodeid", "s_hash")
+        self.eq.push("FS_FILE_OPEN", path=self.tf)
+        self.eq.push("AQ_DOWNLOAD_COMMIT",
+                     share_id="", node_id="nodeid", server_hash="s_hash")
 
         def check1(_):
             self.assertEqual(self.listener.events(), [
@@ -433,7 +460,7 @@ class DownloadFinishedTests(TwistedTestCase):
 
         def check3(_):
             self.assertEqual(self.listener.events(), [
-                        ("HQ_HASH_NEW", self.tf, "hash", "crc", "siz", "stt"),
+                        ("HQ_HASH_NEW", self.tf, "crc", "hash", "siz", "stt"),
                         ("AQ_DOWNLOAD_FINISHED", "", "nodeid", "s_hash"),
             ])
             self.assertFalse(self.tf in self.nanny._blocked)
@@ -442,19 +469,22 @@ class DownloadFinishedTests(TwistedTestCase):
         d = defer.Deferred()
         d.addCallback(check1)
         d.addCallback(lambda _: self.insert_in_hq(self.tf, "nodeid"))
-        d.addCallback(lambda _: self.eq.push("FS_FILE_CLOSE_WRITE", self.tf))
+        d.addCallback(lambda _: self.eq.push("FS_FILE_CLOSE_WRITE",
+                                             path=self.tf))
         d.addCallback(check2)
         d.addCallback(lambda _: self.release_hq())
-        d.addCallback(lambda _: self.eq.push("HQ_HASH_NEW", self.tf, "hash",
-                                             "crc", "siz", "stt"))
+        d.addCallback(lambda _: self.eq.push("HQ_HASH_NEW", path=self.tf,
+                                             hash="hash", crc32="crc",
+                                             size="siz", stat="stt"))
         d.addCallback(check3)
         d.callback(None)
         return d
 
     def test_create_discards(self):
         """The block and open count is discarded when file created."""
-        self.eq.push("FS_FILE_OPEN", self.tf)
-        self.eq.push("AQ_DOWNLOAD_COMMIT", "", "nodeid", "s_hash")
+        self.eq.push("FS_FILE_OPEN", path=self.tf)
+        self.eq.push("AQ_DOWNLOAD_COMMIT",
+                     share_id="", node_id="nodeid", server_hash="s_hash")
 
         def check1(_):
             self.assertEqual(self.listener.events(), [
@@ -473,7 +503,8 @@ class DownloadFinishedTests(TwistedTestCase):
 
         d = defer.Deferred()
         d.addCallback(check1)
-        d.addCallback(lambda _: self.eq.push("FS_FILE_CREATE", self.tf))
+        d.addCallback(lambda _: self.eq.push("FS_FILE_CREATE",
+                                             path=self.tf))
         d.addCallback(check2)
         d.callback(None)
         return d
@@ -486,15 +517,17 @@ class DownloadFinishedTests(TwistedTestCase):
             ])
 
         d = defer.Deferred()
-        d.addCallback(lambda _: self.eq.push("FS_FILE_CREATE", self.tf))
+        d.addCallback(lambda _: self.eq.push("FS_FILE_CREATE",
+                                             path=self.tf))
         d.addCallback(check)
         d.callback(None)
         return d
 
     def test_delete_discards(self):
         """The block and open count is discarded when file deleted."""
-        self.eq.push("FS_FILE_OPEN", self.tf)
-        self.eq.push("AQ_DOWNLOAD_COMMIT", "", "nodeid", "s_hash")
+        self.eq.push("FS_FILE_OPEN", path=self.tf)
+        self.eq.push("AQ_DOWNLOAD_COMMIT",
+                     share_id="", node_id="nodeid", server_hash="s_hash")
 
         def check1(_):
             self.assertEqual(self.listener.events(), [
@@ -513,7 +546,8 @@ class DownloadFinishedTests(TwistedTestCase):
 
         d = defer.Deferred()
         d.addCallback(check1)
-        d.addCallback(lambda _: self.eq.push("FS_FILE_DELETE", self.tf))
+        d.addCallback(lambda _: self.eq.push("FS_FILE_DELETE",
+                                             path=self.tf))
         d.addCallback(check2)
         d.callback(None)
         return d
@@ -526,7 +560,8 @@ class DownloadFinishedTests(TwistedTestCase):
             ])
 
         d = defer.Deferred()
-        d.addCallback(lambda _: self.eq.push("FS_FILE_DELETE", self.tf))
+        d.addCallback(lambda _: self.eq.push("FS_FILE_DELETE",
+                                             path=self.tf))
         d.addCallback(check)
         d.callback(None)
         return d
@@ -539,8 +574,9 @@ class DownloadFinishedTests(TwistedTestCase):
         self.fsm.set_node_id(tf2, "nodeid2")
 
         # initial events
-        self.eq.push("FS_FILE_OPEN", self.tf)
-        self.eq.push("AQ_DOWNLOAD_COMMIT", "", "nodeid", "s_hash")
+        self.eq.push("FS_FILE_OPEN", path=self.tf)
+        self.eq.push("AQ_DOWNLOAD_COMMIT",
+                     share_id="", node_id="nodeid", server_hash="s_hash")
 
         def check1(_):
             self.assertEqual(self.listener.events(), [
@@ -565,9 +601,11 @@ class DownloadFinishedTests(TwistedTestCase):
 
         d = defer.Deferred()
         d.addCallback(check1)
-        d.addCallback(lambda _: self.eq.push("FS_FILE_MOVE", self.tf, tf2))
+        d.addCallback(lambda _: self.eq.push("FS_FILE_MOVE",
+                                             path_from=self.tf, path_to=tf2))
         d.addCallback(check2)
-        d.addCallback(lambda _: self.eq.push("FS_FILE_CLOSE_NOWRITE", tf2))
+        d.addCallback(lambda _: self.eq.push("FS_FILE_CLOSE_NOWRITE",
+                                             path=tf2))
         d.addCallback(check3)
         d.callback(None)
         return d
@@ -582,15 +620,16 @@ class DownloadFinishedTests(TwistedTestCase):
         tf5 = os.path.join(self.usrdir, "fo1234")
 
         dir_from = os.path.join(self.usrdir, "fo12")
-        dir_to = os.path.join(self.usrdir, "bar")
-        newtf2 = os.path.join(self.usrdir, "bar", "tfile")
+        dir_to = os.path.join(self.usrdir, "zar")
+        newtf2 = os.path.join(self.usrdir, "zar", "tfile")
 
         # create them and generate the initial events for all of them
         for i, tf in enumerate((tf1, tf2, tf3, tf4, tf5)):
             self.fsm.create(tf, "")
             self.fsm.set_node_id(tf, "nodeid"+str(i+1))
-            self.eq.push("FS_FILE_OPEN", tf)
-            self.eq.push("AQ_DOWNLOAD_COMMIT", "", "nodeid"+str(i+1), "s_hash")
+            self.eq.push("FS_FILE_OPEN", path=tf)
+            self.eq.push("AQ_DOWNLOAD_COMMIT", share_id="",
+                         node_id="nodeid"+str(i+1), server_hash="s_hash")
 
         def check1(_):
             self.assertEqual(self.listener.events(), [
@@ -636,9 +675,11 @@ class DownloadFinishedTests(TwistedTestCase):
 
         d = defer.Deferred()
         d.addCallback(check1)
-        d.addCallback(lambda _: self.eq.push("FS_DIR_MOVE", dir_from, dir_to))
+        d.addCallback(lambda _: self.eq.push("FS_DIR_MOVE", path_from=dir_from,
+                                             path_to=dir_to))
         d.addCallback(check2)
-        d.addCallback(lambda _: self.eq.push("FS_FILE_CLOSE_NOWRITE", newtf2))
+        d.addCallback(lambda _: self.eq.push("FS_FILE_CLOSE_NOWRITE",
+                                             path=newtf2))
         d.addCallback(check3)
         d.callback(None)
         return d
@@ -649,15 +690,16 @@ class DownloadFinishedTests(TwistedTestCase):
         dir_from = os.path.join(self.usrdir, "foo")
         tf1 = os.path.join(self.usrdir, "foo", "tfile1")
         tf2 = os.path.join(self.usrdir, "foo", "tfile2")
-        dir_to = os.path.join(self.usrdir, "bar")
-        newtf2 = os.path.join(self.usrdir, "bar", "tfile2")
+        dir_to = os.path.join(self.usrdir, "zar")
+        newtf2 = os.path.join(self.usrdir, "zar", "tfile2")
 
         # create the file and generate the initial events
         self.fsm.create(tf1, "")
         self.fsm.set_node_id(tf1, "nodeid")
-        self.eq.push("FS_FILE_OPEN", tf1)
-        self.eq.push("AQ_DOWNLOAD_COMMIT", "", "nodeid", "s_hash")
-        self.eq.push("FS_FILE_OPEN", tf1)
+        self.eq.push("FS_FILE_OPEN", path=tf1)
+        self.eq.push("AQ_DOWNLOAD_COMMIT",
+                     share_id="", node_id="nodeid", server_hash="s_hash")
+        self.eq.push("FS_FILE_OPEN", path=tf1)
 
         def check1(_):
             self.assertEqual(self.listener.events(), [
@@ -704,15 +746,21 @@ class DownloadFinishedTests(TwistedTestCase):
 
         d = defer.Deferred()
         d.addCallback(check1)
-        d.addCallback(lambda _: self.eq.push("FS_FILE_CLOSE_NOWRITE", tf1))
+        d.addCallback(lambda _: self.eq.push("FS_FILE_CLOSE_NOWRITE",
+                                             path=tf1))
         d.addCallback(check2)
-        d.addCallback(lambda _: self.eq.push("FS_FILE_MOVE", tf1, tf2))
+        d.addCallback(lambda _: self.eq.push("FS_FILE_MOVE",
+                                             path_from=tf1, path_to=tf2))
         d.addCallback(check3)
-        d.addCallback(lambda _: self.eq.push("FS_FILE_OPEN", tf2))
-        d.addCallback(lambda _: self.eq.push("FS_DIR_MOVE", dir_from, dir_to))
+        d.addCallback(lambda _: self.eq.push("FS_FILE_OPEN",
+                                             path=tf2))
+        d.addCallback(lambda _: self.eq.push("FS_DIR_MOVE", path_from=dir_from,
+                                             path_to=dir_to))
         d.addCallback(check4)
-        d.addCallback(lambda _: self.eq.push("FS_FILE_CLOSE_NOWRITE", newtf2))
-        d.addCallback(lambda _: self.eq.push("FS_FILE_CLOSE_NOWRITE", newtf2))
+        d.addCallback(lambda _: self.eq.push("FS_FILE_CLOSE_NOWRITE",
+                                             path=newtf2))
+        d.addCallback(lambda _: self.eq.push("FS_FILE_CLOSE_NOWRITE",
+                                             path=newtf2))
         d.addCallback(check5)
         d.callback(None)
         return d
@@ -733,7 +781,7 @@ class DownloadFinishedTests(TwistedTestCase):
 
         def check3(_):
             self.assertEqual(self.listener.events(), [
-                        ("HQ_HASH_NEW", self.tf, "hash", "crc", "siz", "stt"),
+                        ("HQ_HASH_NEW", self.tf, "crc", "hash", "siz", "stt"),
             ])
             self.assertTrue(self.tf in self.nanny._blocked)
 
@@ -745,23 +793,27 @@ class DownloadFinishedTests(TwistedTestCase):
             self.assertFalse(self.tf in self.nanny._blocked)
 
         d = self.insert_in_hq(self.tf, "nodeid")
-        d.addCallback(lambda _: self.eq.push("AQ_DOWNLOAD_COMMIT", "",
-                                             "nodeid", "s_hash"))
+        d.addCallback(lambda _: self.eq.push("AQ_DOWNLOAD_COMMIT", share_id="",
+                                             node_id="nodeid",
+                                             server_hash="s_hash"))
         d.addCallback(check1)
-        d.addCallback(lambda _: self.eq.push("FS_FILE_OPEN", self.tf))
+        d.addCallback(lambda _: self.eq.push("FS_FILE_OPEN", path=self.tf))
         d.addCallback(check2)
         d.addCallback(lambda _: self.release_hq())
-        d.addCallback(lambda _: self.eq.push("HQ_HASH_NEW", self.tf, "hash",
-                                             "crc", "siz", "stt"))
+        d.addCallback(lambda _: self.eq.push("HQ_HASH_NEW", path=self.tf,
+                                             hash="hash", crc32="crc",
+                                             size="siz", stat="stt"))
         d.addCallback(check3)
-        d.addCallback(lambda _: self.eq.push("FS_FILE_CLOSE_NOWRITE", self.tf))
+        d.addCallback(lambda _: self.eq.push("FS_FILE_CLOSE_NOWRITE",
+                                             path=self.tf))
         d.addCallback(check4)
         return d
 
     def test_mixed_close_hash(self):
         """It's ready to release according to open/close, but it's hashing."""
-        self.eq.push("FS_FILE_OPEN", self.tf)
-        self.eq.push("AQ_DOWNLOAD_COMMIT", "", "nodeid", "s_hash")
+        self.eq.push("FS_FILE_OPEN", path=self.tf)
+        self.eq.push("AQ_DOWNLOAD_COMMIT",
+                     share_id="", node_id="nodeid", server_hash="s_hash")
 
         def check1(_):
             self.assertEqual(self.listener.events(), [
@@ -778,7 +830,7 @@ class DownloadFinishedTests(TwistedTestCase):
 
         def check3(_):
             self.assertEqual(self.listener.events(), [
-                        ("HQ_HASH_NEW", self.tf, "hash", "crc", "siz", "stt"),
+                        ("HQ_HASH_NEW", self.tf, "crc", "hash", "siz", "stt"),
                         ("AQ_DOWNLOAD_FINISHED", "", "nodeid", "s_hash"),
             ])
             self.assertFalse(self.tf in self.nanny._blocked)
@@ -786,16 +838,32 @@ class DownloadFinishedTests(TwistedTestCase):
         d = defer.Deferred()
         d.addCallback(check1)
         d.addCallback(lambda _: self.insert_in_hq(self.tf, "nodeid"))
-        d.addCallback(lambda _: self.eq.push("FS_FILE_CLOSE_NOWRITE", self.tf))
+        d.addCallback(lambda _: self.eq.push("FS_FILE_CLOSE_NOWRITE",
+                                             path=self.tf))
         d.addCallback(check2)
         d.addCallback(lambda _: self.release_hq())
-        d.addCallback(lambda _: self.eq.push("HQ_HASH_NEW", self.tf, "hash",
-                                             "crc", "siz", "stt"))
+        d.addCallback(lambda _: self.eq.push("HQ_HASH_NEW", path=self.tf,
+                                             hash="hash", crc32="crc",
+                                             size="siz", stat="stt"))
         d.addCallback(check3)
         d.callback(None)
         return d
 
 
+class EventListenersTests(unittest.TestCase):
+    """Check the event listener API."""
+
+    def test_event_listener(self):
+        """All event listeners should define methods with correct signature."""
+        cls = events_nanny.DownloadFinishedNanny
+        for evtname, evtargs in event_queue.EVENTS.iteritems():
+            meth = getattr(cls, 'handle_' + evtname, None)
+            if meth is not None:
+                defined_args = inspect.getargspec(meth)[0]
+                self.assertEqual(defined_args[0], 'self')
+                self.assertEqual(set(defined_args[1:]), set(evtargs))
+
+
 def test_suite():
     # pylint: disable-msg=C0111
     return unittest.TestLoader().loadTestsFromName(__name__)
