@@ -30,6 +30,7 @@ from twisted.trial.unittest import TestCase as TwistedTestCase
 from twisted.internet import defer, reactor
 
 from contrib.testing import testcase
+from ubuntuone.devtools.handlers import MementoHandler
 from ubuntuone.syncdaemon import hash_queue
 from ubuntuone.storageprotocol.content_hash import \
             content_hash_factory, crc32
@@ -56,9 +57,9 @@ class HasherTests(testcase.BaseTwistedTestCase):
         mark = object()
         queue = hash_queue.UniqueQueue()
         class Helper(object):
-            """helper class"""
-            def push(self, *args):
-                """callback"""
+            """Helper class."""
+            def push(self, event, **kwargs):
+                """Callback."""
         receiver = Helper()
         hasher = hash_queue._Hasher(queue, mark, receiver)
         hasher.start()
@@ -76,22 +77,22 @@ class HasherTests(testcase.BaseTwistedTestCase):
 
     @defer.inlineCallbacks
     def test_called_back_log_ok(self):
-        """Tests that the hasher produces correct info."""
+        """Test that the hasher produces correct info."""
         # create the hasher
         mark = object()
         queue = hash_queue.UniqueQueue()
         d = defer.Deferred()
         class Helper(object):
             """Helper class."""
-            def push(self, *args):
+            def push(self, event, **kwargs):
                 """Callback."""
-                d.callback(args)
+                d.callback(kwargs)
         receiver = Helper()
         hasher = hash_queue._Hasher(queue, mark, receiver)
         hasher.start()
 
         # log config
-        handler = testcase.MementoHandler()
+        handler = MementoHandler()
         handler.setLevel(logging.DEBUG)
         hasher.logger.addHandler(handler)
 
@@ -119,16 +120,16 @@ class HasherTests(testcase.BaseTwistedTestCase):
         hasher.logger.removeHandler(handler)
 
     def test_called_back_ok(self):
-        """Tests that the hasher produces correct info."""
+        """Test that the hasher produces correct info."""
         # create the hasher
         mark = object()
         queue = hash_queue.UniqueQueue()
         d = defer.Deferred()
         class Helper(object):
-            """helper class"""
-            def push(self, *args):
-                """callback"""
-                d.callback(args)
+            """Helper class."""
+            def push(self, event, **kwargs):
+                """Callback."""
+                d.callback((event, kwargs))
         receiver = Helper()
         hasher = hash_queue._Hasher(queue, mark, receiver)
         hasher.start()
@@ -139,23 +140,22 @@ class HasherTests(testcase.BaseTwistedTestCase):
             fh.write("foobar")
         queue.put((testfile, "mdid"))
 
-        def check_info(args):
+        def check_info((event, kwargs)):
             """check the info pushed by the hasher"""
             # pylint: disable-msg=W0612
             hasher.stop()
             hasher.join(timeout=5)
-            event, path, hash, crc, size, stat = args
             self.assertEqual(event, "HQ_HASH_NEW")
             # calculate what we should receive
             realh = content_hash_factory()
             realh.hash_object.update("foobar")
             should_be = realh.content_hash()
             curr_stat = os.stat(testfile)
-            self.assertEquals(should_be, hash)
+            self.assertEquals(should_be, kwargs['hash'])
             for attr in ('st_mode', 'st_ino', 'st_dev', 'st_nlink', 'st_uid',
                          'st_gid', 'st_size', 'st_ctime', 'st_mtime'):
                 self.assertEquals(getattr(curr_stat, attr),
-                                  getattr(stat, attr))
+                                  getattr(kwargs['stat'], attr))
 
 
         d.addCallback(check_info)
@@ -163,16 +163,16 @@ class HasherTests(testcase.BaseTwistedTestCase):
         return d
 
     def test_called_back_error(self):
-        """Tests that the hasher signals error when no file."""
+        """Test that the hasher signals error when no file."""
         # create the hasher
         mark = object()
         queue = hash_queue.UniqueQueue()
         d = defer.Deferred()
         class Helper(object):
-            """helper class"""
-            def push(self, *args):
-                """callback"""
-                d.callback(args)
+            """Helper class."""
+            def push(self, event, **kwargs):
+                """Callback."""
+                d.callback((event, kwargs))
         receiver = Helper()
         hasher = hash_queue._Hasher(queue, mark, receiver)
         hasher.start()
@@ -180,13 +180,12 @@ class HasherTests(testcase.BaseTwistedTestCase):
         # send what to hash
         queue.put(("not_to_be_found", "foo"))
 
-        def check_info(args):
-            """check the info pushed by the hasher"""
+        def check_info((event, kwargs)):
+            """Check the info pushed by the hasher."""
             hasher.stop()
             hasher.join(timeout=5)
-            event, mdid = args
             self.assertEqual(event, "HQ_HASH_ERROR")
-            self.assertEqual(mdid, "foo")
+            self.assertEqual(kwargs['mdid'], "foo")
 
 
         d.addCallback(check_info)
@@ -204,8 +203,9 @@ class HasherTests(testcase.BaseTwistedTestCase):
             tfile = os.path.join(self.test_dir, "tfile"+str(i))
             with open(tfile, "w") as fh:
                 fh.write("supercalifragilistico"+str(i))
-            should_be.append(("HQ_HASH_NEW", tfile, hasher.content_hash(),
-                              crc32(text), len(text), os.stat(tfile)))
+            d = dict(path=tfile, hash=hasher.content_hash(),
+                     crc32=crc32(text), size=len(text), stat=os.stat(tfile))
+            should_be.append(("HQ_HASH_NEW", d))
 
         # create the hasher
         mark = object()
@@ -213,13 +213,13 @@ class HasherTests(testcase.BaseTwistedTestCase):
         d = defer.Deferred()
 
         class Helper(object):
-            """helper class"""
+            """Helper class."""
             # class-closure, cannot use self, pylint: disable-msg=E0213
             def __init__(innerself):
                 innerself.store = []
-            def push(innerself, *args):
-                """callback"""
-                innerself.store.append(args)
+            def push(innerself, event, **kwargs):
+                """Callback."""
+                innerself.store.append((event, kwargs))
                 if len(innerself.store) == 10:
                     hasher.stop()
                     hasher.join(timeout=5)
@@ -254,8 +254,8 @@ class HasherTests(testcase.BaseTwistedTestCase):
         d = defer.Deferred()
 
         class Helper(object):
-            """helper class"""
-            def push(self, event, path, newhash, crc, size, stat):
+            """Helper class."""
+            def push(self, event, path, hash, crc32, size, stat):
                 """callback"""
                 hasher.stop()
                 hasher.join(timeout=5)
@@ -263,7 +263,7 @@ class HasherTests(testcase.BaseTwistedTestCase):
                     d.errback(Exception("envent is not HQ_HASH_NEW"))
                 elif path != testfile:
                     d.errback(Exception("path is not the original one"))
-                elif newhash != testhash:
+                elif hash != testhash:
                     d.errback(Exception("the hashes are different!"))
                 else:
                     d.callback(True)
@@ -296,14 +296,14 @@ class HashQueueTests(testcase.BaseTwistedTestCase):
         testcase.BaseTwistedTestCase.tearDown(self)
 
     def test_called_back_ok(self):
-        """Tests that the hasher produces correct info."""
+        """Test that the hasher produces correct info."""
         # create the hasher
         d = defer.Deferred()
         class Helper(object):
-            """helper class"""
-            def push(self, *args):
-                """callback"""
-                d.callback(args)
+            """Helper class."""
+            def push(self, event, **kwargs):
+                """Callback."""
+                d.callback((event, kwargs))
         receiver = Helper()
         hq = hash_queue.HashQueue(receiver)
         self.addCleanup(hq.shutdown)
@@ -314,49 +314,59 @@ class HashQueueTests(testcase.BaseTwistedTestCase):
             fh.write("foobar")
         hq.insert(testfile, "mdid")
 
-        def check_info(args):
+        def check_info((event, kwargs)):
             """check the info pushed by the hasher"""
             # pylint: disable-msg=W0612
-            event, path, hash, crc, size, stat = args
             self.assertEqual(event, "HQ_HASH_NEW")
             # calculate what we should receive
             realh = content_hash_factory()
             realh.hash_object.update("foobar")
             should_be = realh.content_hash()
             curr_stat = os.stat(testfile)
-            self.assertEquals(should_be, hash)
+            self.assertEquals(should_be, kwargs['hash'])
             for attr in ('st_mode', 'st_ino', 'st_dev', 'st_nlink', 'st_uid',
                          'st_gid', 'st_size', 'st_ctime', 'st_mtime'):
                 self.assertEquals(getattr(curr_stat, attr),
-                                  getattr(stat, attr))
+                                  getattr(kwargs['stat'], attr))
 
         d.addCallback(check_info)
         return d
 
+    @defer.inlineCallbacks
     def test_called_back_error(self):
-        """Tests that the hasher generates an error when no file."""
+        """Test that the hasher generates an error when no file."""
         # create the hasher
         d = defer.Deferred()
         class Helper(object):
-            """helper class"""
-            def push(self, *args):
-                """callback"""
-                d.callback(args)
+            """Helper class."""
+            def push(self, event, **kwargs):
+                """Callback."""
+                d.callback((event, kwargs))
         receiver = Helper()
         hq = hash_queue.HashQueue(receiver)
         self.addCleanup(hq.shutdown)
+        call_later_called = []
+
+        # the call is delayed
+        original_call_later = reactor.callLater
+        def fake_call_later(delay, func, *args, **kwargs):
+            """Fake call later that checks and calls."""
+            if func == reactor.callFromThread:
+                # if it is *our* call, check the args
+                self.assertEqual(delay, .1)
+                self.assertEqual(args, (receiver.push, 'HQ_HASH_ERROR'))
+                self.assertEqual(kwargs, dict(mdid='foo'))
+                call_later_called.append(True)
+            return original_call_later(delay, func, *args, **kwargs)
+        self.patch(reactor, 'callLater', fake_call_later)
 
         # send what to hash
         hq.insert("not_to_be_found", "foo")
 
-        def check_info(args):
-            """check the info pushed by the hasher"""
-            event, mdid = args
-            self.assertEqual(event, "HQ_HASH_ERROR")
-            self.assertEqual(mdid, "foo")
-
-        d.addCallback(check_info)
-        return d
+        event, kwargs = yield d
+        self.assertEqual(event, "HQ_HASH_ERROR")
+        self.assertEqual(kwargs['mdid'], "foo")
+        self.assertTrue(call_later_called)
 
     def test_being_hashed(self):
         """Tell if something is being hashed."""
@@ -366,9 +376,9 @@ class HashQueueTests(testcase.BaseTwistedTestCase):
         open(tfile2, "w").close()
 
         class C(object):
-            """bogus"""
-            def push(self, *a):
-                """none"""
+            """Bogus."""
+            def push(self, e, **k):
+                """None."""
         hq = hash_queue.HashQueue(C())
         self.addCleanup(hq.shutdown)
 
@@ -376,6 +386,7 @@ class HashQueueTests(testcase.BaseTwistedTestCase):
         original_hash = hash_queue._Hasher._hash
 
         def f(*a):
+            """Fake _hash."""
             event.wait()
             return "foo"
         hash_queue._Hasher._hash = f
@@ -408,18 +419,19 @@ class HashQueueTests(testcase.BaseTwistedTestCase):
             tfile = os.path.join(self.test_dir, "tfile"+str(i))
             with open(tfile, "w") as fh:
                 fh.write("supercalifragilistico"+str(i))
-            should_be.append(("HQ_HASH_NEW", tfile, hasher.content_hash(),
-                              crc32(text), len(text), os.stat(tfile)))
+            d = dict(path=tfile, hash=hasher.content_hash(),
+                     crc32=crc32(text), size=len(text), stat=os.stat(tfile))
+            should_be.append(("HQ_HASH_NEW", d))
 
         d = defer.Deferred()
         class Helper(object):
-            """helper class"""
+            """Helper class."""
             # class-closure, cannot use self, pylint: disable-msg=E0213
             def __init__(innerself):
                 innerself.store = []
-            def push(innerself, *args):
-                """callback"""
-                innerself.store.append(args)
+            def push(innerself, event, **kwargs):
+                """Callback."""
+                innerself.store.append((event, kwargs))
                 if len(innerself.store) == 10:
                     if innerself.store[:-1] == should_be[:-1]:
                         d.callback(True)
@@ -447,24 +459,22 @@ class HashQueueTests(testcase.BaseTwistedTestCase):
             tfile = os.path.join(self.test_dir, "tfile"+str(i))
             with open(tfile, "w") as fh:
                 fh.write("supercalifragilistico"+str(i))
-            should_be.append(("HQ_HASH_NEW", tfile, hasher.content_hash(),
-                              crc32(text), len(text), os.stat(tfile)))
+            d = dict(path=tfile, hash=hasher.content_hash(),
+                     crc32=crc32(text), size=len(text), stat=os.stat(tfile))
+            should_be.append(("HQ_HASH_NEW", d))
 
         d = defer.Deferred()
         class Helper(object):
-            """helper class"""
+            """Helper class."""
             # class-closure, cannot use self, pylint: disable-msg=E0213
             def __init__(innerself):
                 innerself.store = []
-            def push(innerself, *args):
-                """callback"""
-                innerself.store.append(args)
+            def push(innerself, event, **kwargs):
+                """Callback."""
+                innerself.store.append((event, kwargs))
                 if len(innerself.store) == 10:
-                    if len(innerself.store[:-1]) == len(should_be[:-1]):
-                        stored = set(innerself.store[:-1])
-                        expected = set(should_be[:-1])
-                        if len(stored.symmetric_difference(expected)) == 0:
-                            d.callback(True)
+                    if innerself.store == should_be:
+                        d.callback(True)
                     else:
                         d.errback(Exception("are different!"))
 
@@ -511,14 +521,14 @@ class HashQueueTests(testcase.BaseTwistedTestCase):
         d = defer.Deferred()
 
         class Helper(object):
-            """helper class"""
-            def push(self, event, path, newhash, crc, size, stat):
-                """callback"""
+            """Helper class."""
+            def push(self, event, path, hash, crc32, size, stat):
+                """Callback."""
                 if event != "HQ_HASH_NEW":
                     d.errback(Exception("envent is not HQ_HASH_NEW"))
                 elif path != testfile:
                     d.errback(Exception("path is not the original one"))
-                elif newhash != testhash:
+                elif hash != testhash:
                     d.errback(Exception("the hashes are different!"))
                 else:
                     d.callback(True)
@@ -569,9 +579,9 @@ class HashQueueTests(testcase.BaseTwistedTestCase):
     def test_shutdown(self):
         """Test that the HashQueue shutdown """
         class Helper(object):
-            """helper class"""
-            def push(self, *args):
-                """callback"""
+            """Helper class."""
+            def push(self, event, **kwargs):
+                """Callback."""
         receiver = Helper()
         hq = hash_queue.HashQueue(receiver)
         hq.shutdown()
@@ -589,9 +599,9 @@ class HashQueueTests(testcase.BaseTwistedTestCase):
             fh.write(testinfo)
 
         class Helper(object):
-            """helper class"""
-            def push(self, *args):
-                """callback"""
+            """Helper class."""
+            def push(self, event, **kwargs):
+                """Callback."""
         receiver = Helper()
         hq = hash_queue.HashQueue(receiver)
         self.addCleanup(hq.shutdown)
@@ -611,9 +621,9 @@ class HashQueueTests(testcase.BaseTwistedTestCase):
     def test_insert_post_shutdown(self):
         """test inserting a path after the shutdown"""
         class Helper(object):
-            """helper class"""
-            def push(self, *args):
-                """callback"""
+            """Helper class."""
+            def push(self, event, **kwargs):
+                """Callback."""
         receiver = Helper()
         hq = hash_queue.HashQueue(receiver)
         hq.shutdown()
