@@ -28,8 +28,10 @@ import uuid
 from twisted.internet import defer, reactor
 
 from contrib.testing import testcase
+from ubuntuone.devtools.handlers import MementoHandler
 from ubuntuone.syncdaemon.local_rescan import LocalRescan
 from ubuntuone.syncdaemon.marker import MDMarker
+from ubuntuone.syncdaemon.tritcask import Tritcask
 from ubuntuone.syncdaemon import (
     event_queue, filesystem_manager, volume_manager
 )
@@ -37,20 +39,22 @@ from ubuntuone.storageprotocol import (
     content_hash as storage_hash, volumes
 )
 
+# our logging level
+TRACE = logging.getLevelName('TRACE')
 
 class FakeEQ(object):
     """Fake EQ"""
     def __init__(self):
         self.pushed = []
 
-    def push(self, event, *args):
+    def push(self, event, **kwargs):
         """Store stuff as pushed."""
-        self.pushed.append((event,) + args)
+        self.pushed.append((event, kwargs))
 
-    def _fake(self, *a):
+    def _fake(self, *a, **k):
         """fake"""
-    inotify_add_watch = inotify_has_watch = freeze_rollback = is_frozen = _fake
-    inotify_rm_watch = freeze_begin = add_to_mute_filter = _fake
+    add_watch = has_watch = freeze_rollback = is_frozen = _fake
+    rm_watch = freeze_begin = add_to_mute_filter = _fake
 
     def freeze_commit(self, events):
         """just store events"""
@@ -65,22 +69,21 @@ class FakeAQ(object):
         self.uploaded = None
         self.downloaded = None
 
-    def upload(self, *args):
+    def upload(self, *args, **kwargs):
         """Store stuff as uploaded."""
-        self.uploaded = args
+        self.uploaded = (args, kwargs)
 
     def download(self, *args):
         """Store stuff as downloaded."""
         self.downloaded = args
 
-    def unlink(self, share_id, parent_id, node_id):
+    def unlink(self, *args):
         """Store stuff as unlinked."""
-        self.unlinked.append((share_id, parent_id, node_id))
+        self.unlinked.append(args)
 
-    def move(self, share_id, node_id, old_parent_id, new_parent_id, new_name):
+    def move(self, *args):
         """Store stuff as moved."""
-        data = (share_id, node_id, old_parent_id, new_parent_id, new_name)
-        self.moved.append(data)
+        self.moved.append(args)
 
 
 class BaseTestCase(testcase.BaseTwistedTestCase):
@@ -95,13 +98,15 @@ class BaseTestCase(testcase.BaseTwistedTestCase):
         usrdir = self.mktemp("usrdir")
         self.fsmdir = self.mktemp("fsmdir")
         self.partials_dir = self.mktemp("partials")
+        self.tritcask_dir = self.mktemp("tritcask")
         # set the home for the tests
         self.old_value = os.environ.get('HOME', None)
         os.environ['HOME'] = self.home_dir
         self.vm = testcase.FakeVolumeManager(usrdir)
+        self.db = Tritcask(self.tritcask_dir)
         self.fsm = filesystem_manager.FileSystemManager(self.fsmdir,
                                                         self.partials_dir,
-                                                        self.vm)
+                                                        self.vm, self.db)
         self.fsm.create(usrdir, "", is_dir=True)
         self.eq = FakeEQ()
         self.fsm.register_eq(self.eq)
@@ -112,21 +117,24 @@ class BaseTestCase(testcase.BaseTwistedTestCase):
             os.environ.pop('HOME')
         else:
             os.environ['HOME'] = self.old_value
+        self.db.shutdown()
         self.rmtree(self.tmpdir)
         testcase.BaseTwistedTestCase.tearDown(self)
 
-    @staticmethod
-    def create_share(share_id, share_name, fsm, shares_dir,
-                     access_level='Modify', accepted=True):
-        """ creates a share """
-        share_path = os.path.join(shares_dir, share_name)
+    @defer.inlineCallbacks
+    def create_share(self, share_id, share_name,
+                     access_level='Modify', accepted=True, subscribed=True):
+        """Create a share."""
+        share_path = os.path.join(self.shares_dir, share_name)
         os.makedirs(share_path)
         share = volume_manager.Share(path=share_path, volume_id=share_id,
                                      access_level=access_level,
-                                     accepted=accepted)
-        fsm.vm.add_share(share)
-        return share
+                                     accepted=accepted,
+                                     subscribed=subscribed)
+        yield self.fsm.vm.add_share(share)
+        defer.returnValue(share)
 
+    @defer.inlineCallbacks
     def create_udf(self, udf_id, node_id, suggested_path, path, subscribed,
                    generation=None, free_bytes=100):
         """Create an UDF and add it to the volume manager."""
@@ -134,8 +142,8 @@ class BaseTestCase(testcase.BaseTwistedTestCase):
                                    suggested_path.decode('utf-8'))
         udf = volume_manager.UDF.from_udf_volume(volume, path)
         udf.subscribed = subscribed
-        self.fsm.vm.add_udf(udf)
-        return udf
+        yield self.fsm.vm.add_udf(udf)
+        defer.returnValue(udf)
 
     def create_node(self, path, is_dir, real=True, which_share=None):
         """Creates a node, really (maybe) and in the metadata."""
@@ -184,11 +192,12 @@ class CollectionTests(BaseTestCase):
         lr.start()
         self.assertEqual(toscan, [self.vm.root.path])
 
+    @defer.inlineCallbacks
     def test_empty_rw(self):
         """Test with one empty Modify share."""
         # create the share
-        share = self.create_share('share_id', 'rw_share',  self.fsm,
-                                       self.shares_dir, access_level='Modify')
+        share = yield self.create_share('share_id', 'rw_share',
+                                        access_level='Modify')
         self.fsm.create(share.path, "share_id", is_dir=True)
         self.fsm.set_node_id(share.path, "uuid")
 
@@ -205,11 +214,12 @@ class CollectionTests(BaseTestCase):
         lr.start()
         self.assertEqual(sorted(toscan), [share.path, self.vm.root.path])
 
+    @defer.inlineCallbacks
     def test_not_empty_rw(self):
         """Test with a Modify share with info."""
         # create the share
-        share = self.create_share('share_id', 'ro_share',  self.fsm,
-                                       self.shares_dir, access_level='Modify')
+        share = yield self.create_share('share_id', 'ro_share',
+                                        access_level='Modify')
         self.fsm.create(share.path, "share_id", is_dir=True)
         self.fsm.set_node_id(share.path, "uuid1")
 
@@ -232,11 +242,12 @@ class CollectionTests(BaseTestCase):
         lr.start()
         self.assertEqual(sorted(toscan), [share.path, self.vm.root.path])
 
+    @defer.inlineCallbacks
     def test_deleted_rw(self):
         """Test with a deleted rw share."""
         # create the share
-        share = self.create_share('share_id', 'rw_share',  self.fsm,
-                                  self.shares_dir, access_level='Modify')
+        share = yield self.create_share('share_id', 'rw_share',
+                                        access_level='Modify')
         self.fsm.create(share.path, "share_id", is_dir=True)
         self.fsm.set_node_id(share.path, "uuid")
 
@@ -261,11 +272,12 @@ class CollectionTests(BaseTestCase):
         self.assertEqual(vol_id, share.volume_id)
         self.assertEqual(sorted(toscan), [self.vm.root.path])
 
+    @defer.inlineCallbacks
     def test_deleted_rw_not_empty(self):
         """Test with a deleted rw share with some nodes in it."""
         # create the share
-        share = self.create_share('share_id', 'rw_share',  self.fsm,
-                                  self.shares_dir, access_level='Modify')
+        share = yield self.create_share('share_id', 'rw_share',
+                                        access_level='Modify')
         self.fsm.create(share.path, "share_id", is_dir=True)
         self.fsm.set_node_id(share.path, "uuid")
 
@@ -302,7 +314,7 @@ class CollectionTests(BaseTestCase):
         """Test with a deleted udf."""
         # create the udf
         udf_path = os.path.expanduser("~/myudf")
-        udf = self.create_udf('udf_id', 'udf_root_node_id', "~/myudf",
+        udf = yield self.create_udf('udf_id', 'udf_root_node_id', "~/myudf",
                                     udf_path, True)
         os.makedirs(udf.path)
         self.fsm.create(udf.path, 'udf_id', is_dir=True)
@@ -332,9 +344,10 @@ class CollectionTests(BaseTestCase):
 class VolumeTestCase(BaseTestCase):
     """Test how LocalRescan manages volumes."""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Init."""
-        BaseTestCase.setUp(self)
+        yield BaseTestCase.setUp(self)
 
         self.env_var = 'HOME'
         self.old_value = os.environ.get(self.env_var, None)
@@ -350,7 +363,8 @@ class VolumeTestCase(BaseTestCase):
             path = os.path.expanduser(suggested_path).encode("utf8")
             os.makedirs(path)
             udf_id, node_id = 'udf_id%i' % i, 'node_id%i' % i
-            udf = self.create_udf(udf_id, node_id, suggested_path, path, True)
+            udf = yield self.create_udf(udf_id, node_id, suggested_path,
+                                        path, True)
             self.volumes.append(udf)
 
             # make FSM aware of it
@@ -488,6 +502,7 @@ class TwistedBase(BaseTestCase):
 
     timeout = 2
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Set up the test."""
         BaseTestCase.setUp(self)
@@ -495,15 +510,16 @@ class TwistedBase(BaseTestCase):
         self.lr = LocalRescan(self.vm, self.fsm, self.eq, self.aq)
 
         # create a share
-        self.share = self.create_share('share_id', 'ro_share',  self.fsm,
-                                       self.shares_dir, access_level='Modify')
+        self.share = yield self.create_share('share_id', 'ro_share',
+                                             access_level='Modify')
         self.fsm.create(self.share.path, "share_id", is_dir=True)
         self.fsm.set_node_id(self.share.path, "uuidshare")
         self.share.node_id = "uuidshare"
         self.vm.shares['share_id'] = self.share
 
-        self.handler = testcase.MementoHandler()
+        self.handler = MementoHandler()
         self._logger = logging.getLogger('ubuntuone.SyncDaemon.local_rescan')
+        self._logger.setLevel(TRACE)
         self._logger.addHandler(self.handler)
 
     def tearDown(self):
@@ -530,14 +546,15 @@ class ComparationTests(TwistedBase):
 
     timeout = 20
 
+    @defer.inlineCallbacks
     def setUp(self):
-        TwistedBase.setUp(self)
+        yield TwistedBase.setUp(self)
 
         # create an udf
         udf_path = os.path.join(self.home_dir, "myudf")
         os.mkdir(udf_path)
-        self.udf = self.create_udf('udf_id', 'udf_root_node_id', "~/myudf",
-                                   udf_path, True)
+        self.udf = yield self.create_udf('udf_id', 'udf_root_node_id',
+                                         "~/myudf", udf_path, True)
         self.fsm.create(self.udf.path, 'udf_id', is_dir=True)
         self.fsm.set_node_id(self.udf.path, 'udf_root_node_id')
 
@@ -550,7 +567,7 @@ class ComparationTests(TwistedBase):
         self.startTest(check)
         return self.deferred
 
-    def test_equals(self):
+    def test_equal_file(self):
         """Test with a share with the same files as metadata."""
         # create a node in the share
         self.create_node("a", is_dir=False)
@@ -558,6 +575,23 @@ class ComparationTests(TwistedBase):
         def check(_):
             """check"""
             self.assertEqual(self.eq.pushed, [])
+            self.assertTrue(self.handler.check(TRACE,
+                            "comp yield", "was here, let's check stat"))
+
+        self.startTest(check)
+        return self.deferred
+
+    def test_equal_dir(self):
+        """Test with a share with the same files as metadata."""
+        # create a node in the share
+        self.create_node("a", is_dir=True)
+
+        def check(_):
+            """check"""
+            self.assertEqual(self.eq.pushed, [])
+            self.assertTrue(self.handler.check(TRACE,
+                            "comp yield", "will be scaned later because "
+                                          "it's in NONE!"))
 
         self.startTest(check)
         return self.deferred
@@ -949,11 +983,12 @@ class ComparationTests(TwistedBase):
         reactor.callLater(.2, self.deferred.callback, None)
         return self.deferred
 
+    @defer.inlineCallbacks
     def test_one_dir_ro_share(self):
         """The dir is in a share that's RO, no error but no action."""
         # create the share
-        share = self.create_share('share_id', 'ro_share2',  self.fsm,
-                                       self.shares_dir, access_level='View')
+        share = yield self.create_share('share_id', 'ro_share2',
+                                        access_level='View')
         self.fsm.create(share.path, "share_id", is_dir=True)
         self.fsm.set_node_id(share.path, "uuidshare")
 
@@ -970,6 +1005,8 @@ class ComparationTests(TwistedBase):
         def check(_):
             """check"""
             self.assertEqual(self.eq.pushed, [('FS_FILE_CLOSE_WRITE', path)])
+            self.assertTrue(self.handler.check_debug("comp yield",
+                                                     "file content changed"))
             self.assertTrue(self.handler.check_debug("differ", "Old", "New"))
 
         self.startTest(check)
@@ -1004,7 +1041,7 @@ class ComparationTests(TwistedBase):
         def check(_):
             """check"""
             self.assertEqual(self.eq.pushed, [
-                ('LR_SCAN_ERROR', "mdid", False),
+                ('LR_SCAN_ERROR', dict(mdid="mdid", udfmode=False)),
             ])
 
         self.deferred.addCallback(check)
@@ -1021,7 +1058,7 @@ class ComparationTests(TwistedBase):
         def check(_):
             """check"""
             self.assertEqual(self.eq.pushed, [
-                ('LR_SCAN_ERROR', "mdid", True),
+                ('LR_SCAN_ERROR', dict(mdid="mdid", udfmode=True)),
             ])
 
         self.deferred.addCallback(check)
@@ -1150,20 +1187,21 @@ class QueuingTests(BaseTestCase):
     """Test that simultaneus calls are queued."""
     timeout = 2
 
+    @defer.inlineCallbacks
     def setUp(self):
         """set up the test."""
-        BaseTestCase.setUp(self)
+        yield BaseTestCase.setUp(self)
         self.deferred = defer.Deferred()
         self.eq = event_queue.EventQueue(self.fsm)
         self.lr = LocalRescan(self.vm, self.fsm, self.eq, self.aq)
 
         # create two shares
-        self.share1 = self.create_share('share_id1', 'ro_share_1',  self.fsm,
-                                       self.shares_dir, access_level='Modify')
+        self.share1 = yield self.create_share('share_id1', 'ro_share_1',
+                                              access_level='Modify')
         self.fsm.create(self.share1.path, "share_id1", is_dir=True)
         self.fsm.set_node_id(self.share1.path, "uuidshare1")
-        self.share2 = self.create_share('share_id2', 'ro_share_2',  self.fsm,
-                                       self.shares_dir, access_level='Modify')
+        self.share2 = yield self.create_share('share_id2', 'ro_share_2',
+                                              access_level='Modify')
         self.fsm.create(self.share2.path, "share_id2", is_dir=True)
         self.fsm.set_node_id(self.share2.path, "uuidshare2")
 
@@ -1574,8 +1612,8 @@ class BadStateTests(TwistedBase):
         self.startTest(check)
         return self.deferred
 
-    def test_LOCAL(self):
-        """We were uploading the file, but it was interrupted."""
+    def test_LOCAL_same(self):
+        """Uploading the file, interrupted, and file is the same."""
         # create the file in metadata and real
         path = os.path.join(self.share.path, "a")
         mdid = self.fsm.create(path, self.share.volume_id, is_dir=False)
@@ -1593,13 +1631,71 @@ class BadStateTests(TwistedBase):
         def check(_):
             """Compare predictable args, and check fh factory."""
             mdobj = self.fsm.get_by_mdid(mdid)
-            self.assertEqual(self.aq.uploaded[:6],
+            self.assertEqual(self.aq.uploaded[0][:7],
                              (mdobj.share_id, mdobj.node_id, mdobj.server_hash,
-                              mdobj.local_hash, mdobj.crc32, mdobj.size))
-            factory = self.aq.uploaded[-1]
+                              mdobj.local_hash, mdobj.crc32, mdobj.size, path))
+            self.assertEqual(self.aq.uploaded[1], {'upload_id':None})
+            factory = self.aq.uploaded[0][-1]
             self.assertTrue(isinstance(factory(), file))
-            self.assertTrue(self.handler.check_info("Resuming upload",
-                                                    "interrupted"))
+            self.assertTrue(self.handler.check_debug("resuming upload",
+                                                     "interrupted"))
+
+        self.startTest(check)
+        return self.deferred
+
+    def test_LOCAL_different(self):
+        """Uploading the file, interrupted, and changed while SD was off."""
+        # create the file in metadata and real
+        path = os.path.join(self.share.path, "a")
+        mdid = self.fsm.create(path, self.share.volume_id, is_dir=False)
+        self.fsm.set_node_id(path, "uuid")
+        with open(path, "w") as fh:
+            fh.write("foo")
+
+        # start a put file, and assume we got interrupted (no server hash)
+        pathhash = self._hash(path)
+        self.fsm.set_by_mdid(mdid, local_hash=pathhash, crc32='foo')
+        mdobj = self.fsm.fs[mdid]
+        mdobj["stat"] = None # stat comparison will fail :)
+        self.fsm.fs[mdid] = mdobj
+
+        def check(_):
+            """Compare predictable args, and check fh factory."""
+            self.assertEqual(self.eq.pushed, [('FS_FILE_CLOSE_WRITE', path)])
+            self.assertTrue(self.handler.check_debug("comp yield", path,
+                                                     "LOCAL and changed"))
+
+        self.startTest(check)
+        return self.deferred
+
+    def test_LOCAL_with_upload_id(self):
+        """We were uploading the file, but it was interrupted."""
+        # create the file in metadata and real
+        path = os.path.join(self.share.path, "a")
+        mdid = self.fsm.create(path, self.share.volume_id, is_dir=False)
+        self.fsm.set_node_id(path, "uuid")
+        with open(path, "w") as fh:
+            fh.write("foo")
+
+        # start a put file, and assume we got interrupted (no server hash)
+        pathhash = self._hash(path)
+        self.fsm.set_by_mdid(mdid, local_hash=pathhash, crc32='foo')
+        mdobj = self.fsm.fs[mdid]
+        mdobj["stat"] = os.lstat(path)
+        mdobj["upload_id"] = 'hola'
+        self.fsm.fs[mdid] = mdobj
+
+        def check(_):
+            """Compare predictable args, and check fh factory."""
+            mdobj = self.fsm.get_by_mdid(mdid)
+            self.assertEqual(self.aq.uploaded[0][:7],
+                             (mdobj.share_id, mdobj.node_id, mdobj.server_hash,
+                              mdobj.local_hash, mdobj.crc32, mdobj.size, path))
+            self.assertEqual(self.aq.uploaded[1], {'upload_id':'hola'})
+            factory = self.aq.uploaded[0][-1]
+            self.assertTrue(isinstance(factory(), file))
+            self.assertTrue(self.handler.check_debug("resuming upload",
+                                                     "interrupted"))
 
         self.startTest(check)
         return self.deferred
@@ -1624,8 +1720,9 @@ class BadStateTests(TwistedBase):
         def check(_):
             """Compare predictable args, and check fh factory."""
             mdobj = self.fsm.get_by_mdid(mdid)
-            self.assertEqual(self.aq.downloaded[:3], (
-                             mdobj.share_id, mdobj.node_id, mdobj.server_hash))
+            self.assertEqual(self.aq.downloaded[:4],
+                             (mdobj.share_id, mdobj.node_id,
+                              mdobj.server_hash, path))
             factory = self.aq.downloaded[-1]
             self.assertTrue(isinstance(factory(), file))
             self.assertTrue(self.handler.check_debug("comp yield", "SERVER"))
@@ -1651,8 +1748,9 @@ class BadStateTests(TwistedBase):
         def check(_):
             """Compare predictable args, and check fh factory."""
             mdobj = self.fsm.get_by_mdid(mdid)
-            self.assertEqual(self.aq.downloaded[:3], (
-                             mdobj.share_id, mdobj.node_id, mdobj.server_hash))
+            self.assertEqual(self.aq.downloaded[:4],
+                             (mdobj.share_id, mdobj.node_id,
+                              mdobj.server_hash, path))
             factory = self.aq.downloaded[-1]
             self.assertTrue(isinstance(factory(), file))
             self.assertTrue(self.handler.check_debug("comp yield", "SERVER"))
@@ -2000,11 +2098,12 @@ class BadStateTests(TwistedBase):
         self.startTest(check)
         return self.deferred
 
+    @defer.inlineCallbacks
     def test_SERVER_file_ro_share(self):
         """We were downloading the file, but it was interrupted in RO share."""
         # create the file in metadata
-        ro_share = self.create_share('share_ro_id', 'share_ro2',  self.fsm,
-                                       self.shares_dir, access_level='View')
+        ro_share = yield self.create_share('share_ro_id', 'share_ro2',
+                                           access_level='View')
         self.fsm.create(ro_share.path, ro_share.id, is_dir=True)
         self.fsm.set_node_id(ro_share.path, "uuidshare")
         path = os.path.join(ro_share.path, "a")
@@ -2025,23 +2124,25 @@ class BadStateTests(TwistedBase):
         def check(_):
             """Compare predictable args, and check fh factory."""
             mdobj = self.fsm.get_by_mdid(mdid)
-            self.assertEqual(self.aq.downloaded[:3], (
-                             mdobj.share_id, mdobj.node_id, mdobj.server_hash))
+            self.assertEqual(self.aq.downloaded[:4],
+                             (mdobj.share_id, mdobj.node_id,
+                              mdobj.server_hash, path))
             factory = self.aq.downloaded[-1]
             self.assertTrue(isinstance(factory(), file))
             self.assertTrue(self.handler.check_debug("comp yield", "SERVER"))
 
         self.startTest(check)
-        return self.deferred
+        yield self.deferred
 
+    @defer.inlineCallbacks
     def test_SERVER_dir_ro_share(self):
         """Found a dir in SERVER in a ro_share.
 
         This was valid before, but no more, so we just fix and log in warning.
         """
         # create the dir in metadata
-        ro_share = self.create_share('share_ro_id', 'share_ro2',  self.fsm,
-                                       self.shares_dir, access_level='View')
+        ro_share = yield self.create_share('share_ro_id', 'share_ro2',
+                                           access_level='View')
         self.fsm.create(ro_share.path, ro_share.id, is_dir=True)
         self.fsm.set_node_id(ro_share.path, "uuidshare")
 
@@ -2070,7 +2171,7 @@ class BadStateTests(TwistedBase):
                                                 "Found a directory in SERVER"))
 
         self.startTest(check)
-        return self.deferred
+        yield self.deferred
 
     def test_check_stat_None(self):
         """Test check_stat with oldstat = None."""
@@ -2128,26 +2229,28 @@ class RootBadStateTests(TwistedBase):
         self.fsm.set_node_id(self.vm.root.path, self.vm.root.node_id)
         return self._test_it(self.vm.root)
 
+    @defer.inlineCallbacks
     def test_SERVER_share(self):
         """We were downloading the share root dir but it was interrupted."""
         # create a share
-        share = self.create_share('share_id_1', 'rw_share',  self.fsm,
-                                       self.shares_dir, access_level='Modify')
+        share = yield self.create_share('share_id_1', 'rw_share',
+                                        access_level='Modify')
         self.fsm.create(share.path, "share_id_1", is_dir=True)
         self.fsm.set_node_id(share.path, "uuid_share_1")
         share.node_id = "uuid_share_1"
         self.vm.shares['share_id_1'] = share
-        return self._test_it(share)
+        yield self._test_it(share)
 
+    @defer.inlineCallbacks
     def test_SERVER_udf(self):
         """We were downloading the udf root dir, but it was interrupted."""
         udf_path = os.path.join(self.home_dir, "myudf")
         os.mkdir(udf_path)
-        udf = self.create_udf('udf_id', 'udf_root_node_id', "~/myudf",
-                                   udf_path, True)
+        udf = yield self.create_udf('udf_id', 'udf_root_node_id', "~/myudf",
+                                    udf_path, True)
         self.fsm.create(udf.path, 'udf_id', is_dir=True)
         self.fsm.set_node_id(udf.path, 'udf_root_node_id')
-        return self._test_it(udf)
+        yield self._test_it(udf)
 
 
 class LimboTests(TwistedBase):
@@ -2173,8 +2276,9 @@ class LimboTests(TwistedBase):
         def check(_):
             """Check."""
             self.assertEqual(self.aq.moved, [])
-            self.assertEqual(self.aq.unlinked,
-                             [(self.share.volume_id, "parent_id", "uuid")])
+            self.assertEqual(self.aq.unlinked, [(self.share.volume_id,
+                                                 "parent_id", "uuid", path,
+                                                 True)])
             self.assertTrue(self.handler.check_info(
                             "generating Unlink from trash"))
 
@@ -2183,22 +2287,23 @@ class LimboTests(TwistedBase):
 
     def test_trash_two(self):
         """Two nodes (file and dir) in the trash."""
-        path = os.path.join(self.share.path, "a")
-        mdid1 = self.fsm.create(path, self.share.volume_id, is_dir=True)
-        self.fsm.set_node_id(path, "uuid1")
+        path1 = os.path.join(self.share.path, "a")
+        mdid1 = self.fsm.create(path1, self.share.volume_id, is_dir=True)
+        self.fsm.set_node_id(path1, "uuid1")
         self.fsm.delete_to_trash(mdid1, "parent_id")
 
-        path = os.path.join(self.share.path, "b")
-        mdid2 = self.fsm.create(path, self.share.volume_id, is_dir=False)
-        self.fsm.set_node_id(path, "uuid2")
+        path2 = os.path.join(self.share.path, "b")
+        mdid2 = self.fsm.create(path2, self.share.volume_id, is_dir=False)
+        self.fsm.set_node_id(path2, "uuid2")
         self.fsm.delete_to_trash(mdid2, "parent_id")
 
         def check(_):
             """Check."""
             self.assertEqual(self.aq.moved, [])
-            self.assertEqual(sorted(self.aq.unlinked),
-                             [(self.share.volume_id, "parent_id", "uuid1"),
-                              (self.share.volume_id, "parent_id", "uuid2")])
+            self.assertEqual(sorted(self.aq.unlinked), [
+                (self.share.volume_id, "parent_id", "uuid1", path1, True),
+                (self.share.volume_id, "parent_id", "uuid2", path2, False),
+            ])
 
         self.startTest(check)
         return self.deferred
@@ -2255,13 +2360,14 @@ class LimboTests(TwistedBase):
     def test_move_limbo_one(self):
         """Something in the move_limbo."""
         self.fsm.add_to_move_limbo("share", "uuid", "old_parent",
-                                   "new_parent", "new_name")
+                                   "new_parent", "new_name", "p_from", "p_to")
 
         def check(_):
             """Check."""
             self.assertEqual(self.aq.unlinked, [])
-            self.assertEqual(self.aq.moved, [("share", "uuid", "old_parent",
-                                             "new_parent", "new_name")])
+            self.assertEqual(self.aq.moved,
+                             [("share", "uuid", "old_parent",
+                               "new_parent", "new_name", "p_from", "p_to")])
             self.assertTrue(self.handler.check_info(
                             "generating Move from limbo"))
 
@@ -2270,15 +2376,18 @@ class LimboTests(TwistedBase):
 
     def test_move_limbo_two(self):
         """Two nodes (file and dir) in the move_limbo."""
-        n1 = ("share1", "uuid1", "old_parent1", "new_parent1", "new_name1")
-        n2 = ("share2", "uuid2", "old_parent2", "new_parent2", "new_name2")
-        self.fsm.add_to_move_limbo(*n1)
-        self.fsm.add_to_move_limbo(*n2)
+        self.fsm.add_to_move_limbo("s1", "u1", "op1", "np1", "n1",
+                                   "p_from", "p_to")
+        self.fsm.add_to_move_limbo("s2", "u2", "op2", "np2", "n2",
+                                   "p_from", "p_to")
 
         def check(_):
             """Check."""
             self.assertEqual(self.aq.unlinked, [])
-            self.assertEqual(sorted(self.aq.moved), [n1, n2])
+            self.assertEqual(sorted(self.aq.moved), [
+                ("s1", "u1", "op1", "np1", "n1", "p_from", "p_to"),
+                ("s2", "u2", "op2", "np2", "n2", "p_from", "p_to"),
+            ])
 
         self.startTest(check)
         return self.deferred
@@ -2291,14 +2400,16 @@ class LimboTests(TwistedBase):
         self.fsm.delete_to_trash(mdid, "parent_id")
 
         self.fsm.add_to_move_limbo("share", "uuid", "old_parent",
-                                   "new_parent", "new_name")
+                                   "new_parent", "new_name", "p_from", "p_to")
 
         def check(_):
             """Check."""
-            self.assertEqual(self.aq.moved, [("share", "uuid", "old_parent",
-                                             "new_parent", "new_name")])
-            self.assertEqual(self.aq.unlinked,
-                             [(self.share.volume_id, "parent_id", "uuid")])
+            self.assertEqual(self.aq.moved,
+                             [("share", "uuid", "old_parent", "new_parent",
+                               "new_name", "p_from", "p_to")])
+            self.assertEqual(self.aq.unlinked, [(self.share.volume_id,
+                                                 "parent_id", "uuid", path,
+                                                 True)])
 
         self.startTest(check)
         return self.deferred
@@ -2306,10 +2417,10 @@ class LimboTests(TwistedBase):
     def test_move_limbo_markers(self):
         """Trash with node_id in None."""
         # load 4 things in move limbo, each with a MDMarker (not for 'name')
-        n = ("share_id", "node_id", "old_parent_id", "new_parent_id", "name")
+        n = ("sh", "node", "old_parent", "new_parent", "name", "pfrom", "pto")
         for i in range(4):
             m = list(n)
-            m[1] = 'node_id_%d' % i
+            m[1] = 'node_%d' % i
             m[i] = MDMarker(m[i])
         self.fsm.add_to_move_limbo(*m)
 
@@ -2329,16 +2440,17 @@ class ParentWatchForUDFTestCase(BaseTestCase):
 
     timeout = 2
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Init."""
-        BaseTestCase.setUp(self)
+        yield BaseTestCase.setUp(self)
         self._deferred = defer.Deferred()
         self.eq = event_queue.EventQueue(self.fsm)
-        self.original_add = self.eq.inotify_add_watch
-        self.original_has = self.eq.inotify_has_watch
+        self.original_add = self.eq.add_watch
+        self.original_has = self.eq.has_watch
         self.watches = []
-        self.eq.inotify_add_watch = lambda path: self.watches.append(path)
-        self.eq.inotify_has_watch = lambda path: path in self.watches
+        self.eq.add_watch = lambda path: self.watches.append(path)
+        self.eq.has_watch = lambda path: path in self.watches
 
         self.lr = LocalRescan(self.vm, self.fsm, self.eq, self.aq)
 
@@ -2351,17 +2463,23 @@ class ParentWatchForUDFTestCase(BaseTestCase):
         path = os.path.expanduser(suggested_path).encode("utf8")
         os.makedirs(path)
         udf_id, node_id = 'udf_id', 'node_id'
-        self.udf = self.create_udf(udf_id, node_id, suggested_path, path, True)
+        self.udf = yield self.create_udf(udf_id, node_id, suggested_path,
+                                         path, True)
         self.ancestors = self.udf.ancestors # need a fake HOME
 
         # make FSM aware of it
         self.fsm.create(self.udf.path, udf_id, is_dir=True)
         self.fsm.set_node_id(self.udf.path, node_id)
 
+        # logging
+        self.handler = MementoHandler()
+        self._logger = logging.getLogger('ubuntuone.SyncDaemon')
+        self._logger.addHandler(self.handler)
+
     def tearDown(self):
         """Cleanup."""
-        self.eq.inotify_add_watch = self.original_add
-        self.eq.inotify_has_watch = self.original_has
+        self.eq.add_watch = self.original_add
+        self.eq.has_watch = self.original_has
 
         self.eq.shutdown()
         self.eq = None
@@ -2371,6 +2489,8 @@ class ParentWatchForUDFTestCase(BaseTestCase):
         else:
             os.environ[self.env_var] = self.old_value
 
+        self._logger.removeHandler(self.handler)
+
         BaseTestCase.tearDown(self)
 
     def test_ancestors_have_watch(self):
@@ -2391,25 +2511,36 @@ class ParentWatchForUDFTestCase(BaseTestCase):
                       (expected, actual, difference)
                 self._deferred.errback(msg)
 
+        def check_log(_):
+            """Check log."""
+            self.assertTrue(self.handler.check_debug("Adding watch to UDF's",
+                                                     self.ancestors[0]))
+
         d = self.lr.start()
         d.addCallback(ancestors_have_watch)
+        d.addCallback(check_log)
         return self._deferred
 
     def test_watch_is_not_added_if_present(self):
         """Watches are not added if present."""
 
         for path in self.ancestors:
-            self.eq.inotify_add_watch(path)
+            self.eq.add_watch(path)
 
         def no_watch_added(_):
             """Check."""
             for path in self.udf.ancestors:
                 self.assertEquals(1, self.watches.count(path))
 
+        def check_log(_):
+            """Check log."""
+            self.assertFalse(self.handler.check_debug("Adding watch to UDF's"))
+
         d = self.lr.start()
         d.addCallback(no_watch_added)
         d.addCallback(lambda _:
             self._deferred.callback(None))
+        d.addCallback(check_log)
         return self._deferred
 
 
