@@ -20,14 +20,14 @@ from __future__ import with_statement
 
 import logging
 import unittest
-from contrib.testing.testcase import MementoHandler
+from ubuntuone.devtools.handlers import MementoHandler
 from ubuntuone.syncdaemon.logger import (
     DebugCapture,
     NOTE,
     TRACE,
     root_logger,
     twisted_logger,
-    inotify_logger,
+    filesystem_logger,
     MultiFilter,
 )
 
@@ -223,14 +223,14 @@ class FilterTests(unittest.TestCase):
         self.handler = MementoHandler()
         self.handler.setLevel(logging.DEBUG)
         root_logger.addHandler(self.handler)
-        inotify_logger.addHandler(self.handler)
+        filesystem_logger.addHandler(self.handler)
         twisted_logger.addHandler(self.handler)
 
     def tearDown(self):
         """close the handler and restore the logger (Logger's are global)"""
         self.handler.close()
         root_logger.removeHandler(self.handler)
-        inotify_logger.removeHandler(self.handler)
+        filesystem_logger.removeHandler(self.handler)
         twisted_logger.removeHandler(self.handler)
 
     def test_multiple_filters(self):
