@@ -46,20 +46,14 @@ class FakeRequestQueue(object):
     It has the behaviour of the real queues regarding waiting, done,
     and counter.
 
-    - WAITING when command is queued and the queue was empty.
-    - WAITING when command is done and queue still has stuff
+    - WAITING when command is queued and the queue was empty (first cmd, bah).
     - DONE when command is done and queue is empty
 
     It does NOT send a WAITING everytime something is queued, and it does
     NOT send a DONE everytime a command finishes.
     """
-    def __init__(self, name, execs):
-        self.name = name
-        if name == 'MQ':
-            self.event_preffix = 'SYS_META_QUEUE_'
-        else:
-            self.event_preffix = 'SYS_CONTENT_QUEUE_'
-        self.execs = execs
+    def __init__(self):
+        self.active = False
         self.jobs = []
 
     def __len__(self):
@@ -69,33 +63,21 @@ class FakeRequestQueue(object):
         """Inserts a job."""
         self.jobs.append(job)
         if len(self) == 1:
-            self.qm.on_event(self.event_preffix + 'WAITING')
+            self.qm.on_event('SYS_QUEUE_WAITING')
 
     def run(self):
-        """Executes one job."""
-        if not len(self):
-            raise ValueError("Run without jobs! (%s)" % self.jobs)
+        """Run."""
+        self.active = True
 
-        def executed(_):
-            if self.jobs:
-                self.qm.on_event(self.event_preffix + 'WAITING')
-            else:
-                self.qm.on_event(self.event_preffix + 'DONE')
-
-        self.execs.append(self.name)
-        job = self.jobs.pop(0)
-        job.addCallback(executed)
+    def stop(self):
+        """Stop."""
+        self.active = False
 
 
 class FakeActionQueue(object):
     """Fake class to log actions on AQ."""
     def __init__(self):
-        # this '.execs' is not in the real AQ, it's used in both queues to
-        # record themselves in the same list; this way we can assert over
-        # the order and amount of queue executions
-        self.execs = []
-        self.meta_queue = FakeRequestQueue('MQ', self.execs)
-        self.content_queue = FakeRequestQueue('CQ', self.execs)
+        self.queue = FakeRequestQueue()
         self.actions = []
 
     def __getattribute__(self, name):
@@ -124,6 +106,7 @@ class FakeMain(object):
 
 class Base(TwistedTestCase):
     """Base class for state tests."""
+
     def setUp(self):
         # create fake classes
         self.eq = FakeEventQueue()
@@ -134,8 +117,7 @@ class Base(TwistedTestCase):
         self.cm = self.sm.connection
 
         # add QM to queues, to push events for testing
-        self.aq.content_queue.qm = self.qm
-        self.aq.meta_queue.qm = self.qm
+        self.aq.queue.qm = self.qm
 
         # add logger
         self.fake_logger = FakeLogger()
@@ -178,15 +160,12 @@ class Base(TwistedTestCase):
     def tearDown(self):
         self.sm.connection.shutdown()
 
+
 class QueueBase(Base):
     """Basic setup for QueueManager."""
 
     def setUp(self):
         Base.setUp(self)
-
-        # set up everything so run() will actually run the queues
-        self.qm.cq.queue(defer.Deferred())
-        self.qm.mq.queue(defer.Deferred())
         self.sm.state = StateManager.QUEUE_MANAGER
 
 
@@ -202,311 +181,43 @@ class TestQueueManagerTransitions(QueueBase):
         m = "Bad Event received: Got %r while in %s" % (event, n_from)
         self.assertEqual(in_log, self.check_log('warning', m))
 
-    def test_idle_mq_waiting(self):
-        """MQ Waiting when in IDLE."""
-        self.check('IDLE', 'SYS_META_QUEUE_WAITING', 'WORKING_ON_METADATA')
-        self.assertEqual(self.aq.execs, ['MQ'])
+    def test_idle_waiting(self):
+        """Waiting when in IDLE."""
+        self.check('IDLE', 'SYS_QUEUE_WAITING', 'WORKING')
 
-    def test_idle_mq_done(self):
-        """MQ Done when in IDLE."""
-        self.check('IDLE', 'SYS_META_QUEUE_DONE', 'IDLE', in_log=True)
+    def test_idle_done(self):
+        """Done when in IDLE."""
+        self.check('IDLE', 'SYS_QUEUE_DONE', 'IDLE', in_log=True)
 
-    def test_idle_cq_waiting(self):
-        """CQ Waiting when in IDLE."""
-        self.check('IDLE', 'SYS_CONTENT_QUEUE_WAITING', 'WORKING_ON_CONTENT')
-        self.assertEqual(self.aq.execs, ['CQ'])
+    def test_working_waiting(self):
+        """Waiting when in WORKING."""
+        self.check('WORKING', 'SYS_QUEUE_WAITING', 'WORKING', in_log=True)
 
-    def test_idle_cq_done(self):
-        """CQ Done when in IDLE."""
-        self.check('IDLE', 'SYS_CONTENT_QUEUE_DONE', 'IDLE', in_log=True)
-
-    def test_workingmetadata_mq_waiting(self):
-        """MQ Waiting when in WORKING_ON_METADATA."""
-        self.check('WORKING_ON_METADATA', 'SYS_META_QUEUE_WAITING',
-                   'WORKING_ON_METADATA')
-        self.assertEqual(self.aq.execs, ['MQ'])
-
-    def test_workingmetadata_mq_done(self):
-        """MQ Done when in WORKING_ON_METADATA."""
-        self.check('WORKING_ON_METADATA', 'SYS_META_QUEUE_DONE', 'IDLE')
-
-    def test_workingmetadata_cq_waiting(self):
-        """CQ Waiting when in WORKING_ON_METADATA."""
-        self.check('WORKING_ON_METADATA', 'SYS_CONTENT_QUEUE_WAITING',
-                   'WORKING_ON_BOTH')
-
-    def test_workingmetadata_cq_done(self):
-        """CQ Done when in WORKING_ON_METADATA."""
-        self.check('WORKING_ON_METADATA', 'SYS_CONTENT_QUEUE_DONE',
-                   'WORKING_ON_METADATA', in_log=True)
-
-    def test_workingcontent_mq_waiting(self):
-        """MQ Waiting when in WORKING_ON_CONTENT."""
-        self.check('WORKING_ON_CONTENT', 'SYS_META_QUEUE_WAITING',
-                   'WORKING_ON_BOTH')
-        self.assertEqual(self.aq.execs, ['MQ'])
-
-    def test_workingcontent_mq_done(self):
-        """MQ Done when in WORKING_ON_CONTENT."""
-        self.check('WORKING_ON_CONTENT', 'SYS_META_QUEUE_DONE',
-                   'WORKING_ON_CONTENT', in_log=True)
-
-    def test_workingcontent_cq_waiting(self):
-        """CQ Waiting when in WORKING_ON_CONTENT."""
-        self.check('WORKING_ON_CONTENT', 'SYS_CONTENT_QUEUE_WAITING',
-                   'WORKING_ON_CONTENT')
-        self.assertEqual(self.aq.execs, ['CQ'])
-
-    def test_workingcontent_cq_done(self):
-        """CQ Done when in WORKING_ON_CONTENT."""
-        self.check('WORKING_ON_CONTENT', 'SYS_CONTENT_QUEUE_DONE', 'IDLE')
-
-    def test_workingboth_mq_waiting(self):
-        """MQ Waiting when in WORKING_ON_BOTH."""
-        self.check('WORKING_ON_BOTH', 'SYS_META_QUEUE_WAITING',
-                   'WORKING_ON_BOTH')
-        self.assertEqual(self.aq.execs, ['MQ'])
-
-    def test_workingboth_mq_done(self):
-        """MQ Done when in WORKING_ON_BOTH."""
-        self.check('WORKING_ON_BOTH', 'SYS_META_QUEUE_DONE',
-                   'WORKING_ON_CONTENT')
-        self.assertEqual(self.aq.execs, ['CQ'])
-
-    def test_workingboth_cq_waiting(self):
-        """CQ Waiting when in WORKING_ON_BOTH."""
-        self.check('WORKING_ON_BOTH', 'SYS_CONTENT_QUEUE_WAITING',
-                   'WORKING_ON_BOTH')
-
-    def test_workingboth_cq_done(self):
-        """CQ Done when in WORKING_ON_BOTH."""
-        self.check('WORKING_ON_BOTH', 'SYS_CONTENT_QUEUE_DONE',
-                   'WORKING_ON_METADATA')
+    def test_working_done(self):
+        """Done when in WORKING."""
+        self.check('WORKING', 'SYS_QUEUE_DONE', 'IDLE')
 
 
 class TestQueueManagerFromOutside(QueueBase):
-    """Getting into QueueManager."""
+    """Getting in/out QueueManager."""
 
-    def test_idle(self):
+    def test_into_idle(self):
         """Entering when IDLE."""
+        assert not self.aq.queue.active
         self.qm.state = QueueManager.IDLE
         self.sm.state = StateManager.SERVER_RESCAN
         self.sm.handle_default('SYS_SERVER_RESCAN_DONE')
         self.assertEqual(self.qm.state, QueueManager.IDLE)
-        self.assertEqual(self.aq.execs, [])
+        self.assertTrue(self.aq.queue.active)
 
-    def test_workingmeta(self):
-        """Entering when WORKING_ON_METADATA."""
-        self.qm.state = QueueManager.WORKING_ON_METADATA
+    def test_into_working(self):
+        """Entering when WORKING."""
+        assert not self.aq.queue.active
+        self.qm.state = QueueManager.WORKING
         self.sm.state = StateManager.SERVER_RESCAN
         self.sm.handle_default('SYS_SERVER_RESCAN_DONE')
-        self.assertEqual(self.qm.state, QueueManager.WORKING_ON_METADATA)
-        self.assertEqual(self.aq.execs, ['MQ'])
-
-    def test_workingcontent(self):
-        """Entering when WORKING_ON_CONTENT."""
-        self.qm.state = QueueManager.WORKING_ON_CONTENT
-        self.sm.state = StateManager.SERVER_RESCAN
-        self.sm.handle_default('SYS_SERVER_RESCAN_DONE')
-        self.assertEqual(self.qm.state, QueueManager.WORKING_ON_CONTENT)
-        self.assertEqual(self.aq.execs, ['CQ'])
-
-    def test_workingboth(self):
-        """Entering when WORKING_ON_BOTH."""
-        self.qm.state = QueueManager.WORKING_ON_BOTH
-        self.sm.state = StateManager.SERVER_RESCAN
-        self.sm.handle_default('SYS_SERVER_RESCAN_DONE')
-        self.assertEqual(self.qm.state, QueueManager.WORKING_ON_BOTH)
-        self.assertEqual(self.aq.execs, ['MQ'])
-
-
-class TestQueueManagerRunQueues(QueueBase):
-    """QueueManager run the queues when it should."""
-
-    def test_mq_run_something_yes(self):
-        """MQ.run() is executed when something in the queue and working."""
-        self.sm.state = StateManager.QUEUE_MANAGER
-        self.qm._run(self.qm.mq)
-        self.assertEqual(self.aq.execs, ['MQ'])
-
-    def test_cq_run_something_yes(self):
-        """CQ.run() is executed when something in the queue and working."""
-        self.sm.state = StateManager.QUEUE_MANAGER
-        self.qm._run(self.qm.cq)
-        self.assertEqual(self.aq.execs, ['CQ'])
-
-    def test_mq_run_something_noqueue(self):
-        """MQ.run() is not executed if nothing in the queue."""
-        self.qm.mq.jobs[:] = []
-        self.sm.state = StateManager.QUEUE_MANAGER
-        self.qm._run(self.qm.mq)
-        self.assertEqual(self.aq.execs, [])
-
-    def test_cq_run_something_noqueue(self):
-        """CQ.run() is not executed if nothing in the queue."""
-        self.qm.cq.jobs[:] = []
-        self.sm.state = StateManager.QUEUE_MANAGER
-        self.qm._run(self.qm.cq)
-        self.assertEqual(self.aq.execs, [])
-
-    def test_mq_run_something_noworking(self):
-        """MQ.run() is not executed if not working."""
-        self.sm.state = StateManager.INIT  # just not QM
-        self.qm._run(self.qm.mq)
-        self.assertEqual(self.aq.execs, [])
-
-    def test_cq_run_something_noworking(self):
-        """CQ.run() is not executed if not working."""
-        self.sm.state = StateManager.INIT  # just not QM
-        self.qm._run(self.qm.cq)
-        self.assertEqual(self.aq.execs, [])
-
-
-class TestQueueManagerBehaviour(Base):
-    """Behaviour of QueueManager when managing queues.
-
-    The rules of who is sending us events are defined in the FakeQueues above.
-    """
-
-    def setUp(self):
-        Base.setUp(self)
-        self.sm.state = StateManager.QUEUE_MANAGER
-
-    def execute_test(self, events, should_happen):
-        """Execute the test."""
-        for evt in events:
-            self.qm.on_event(evt)
-        self.assertEqual(self.aq.execs, should_happen)
-        self.assertEqual(self.qm.state, QueueManager.IDLE)
-
-    def test_onlyMQ(self):
-        """Only MQ."""
-        self.qm.mq.queue(defer.succeed(True))
-        self.assertEqual(self.aq.execs, ['MQ'])
-        self.assertEqual(self.qm.state, QueueManager.IDLE)
-
-    def test_onlyCQ(self):
-        """Only CQ."""
-        self.qm.cq.queue(defer.succeed(True))
-        self.assertEqual(self.aq.execs, ['CQ'])
-        self.assertEqual(self.qm.state, QueueManager.IDLE)
-
-    def test_mixed_simple(self):
-        """A couple of both, sequential."""
-        self.qm.mq.queue(defer.succeed(True))
-        self.qm.cq.queue(defer.succeed(True))
-        self.qm.mq.queue(defer.succeed(True))
-        self.qm.cq.queue(defer.succeed(True))
-        self.assertEqual(self.aq.execs, ['MQ', 'CQ', 'MQ', 'CQ'])
-        self.assertEqual(self.qm.state, QueueManager.IDLE)
-
-    def test_workingCQ_stillCQ(self):
-        """Has CQ pending when finishing CQ."""
-        d = defer.Deferred()
-        self.qm.cq.queue(d)
-        self.qm.cq.queue(defer.succeed(True))
-        d.callback(True)
-
-        self.assertEqual(self.aq.execs, ['CQ', 'CQ'])
-        self.assertEqual(self.qm.state, QueueManager.IDLE)
-
-    def test_workingCQ_stillMQ_first_ends_CQ(self):
-        """Has MQ pending when finishing CQ, then CQ finishes first."""
-        dc = defer.Deferred()
-        dm = defer.Deferred()
-        self.qm.cq.queue(dc)
-        self.qm.mq.queue(dm)
-        dc.callback(True)
-        dm.callback(True)
-
-        self.assertEqual(self.aq.execs, ['CQ', 'MQ'])
-        self.assertEqual(self.qm.state, QueueManager.IDLE)
-
-    def test_workingCQ_stillMQ_first_ends_MQ(self):
-        """Has MQ pending when finishing CQ, then MQ finishes first."""
-        dc = defer.Deferred()
-        dm = defer.Deferred()
-        self.qm.cq.queue(dc)
-        self.qm.mq.queue(dm)
-        dm.callback(True)
-        dc.callback(True)
-
-        self.assertEqual(self.aq.execs, ['CQ', 'MQ'])
-        self.assertEqual(self.qm.state, QueueManager.IDLE)
-
-    def test_workingMQ_stillCQ(self):
-        """Has CQ pending when finishing MQ."""
-        d = defer.Deferred()
-        self.qm.mq.queue(d)
-        self.qm.cq.queue(defer.succeed(True))
-        d.callback(True)
-
-        self.assertEqual(self.aq.execs, ['MQ', 'CQ'])
-        self.assertEqual(self.qm.state, QueueManager.IDLE)
-
-    def test_workingMQ_stillMQ(self):
-        """Has MQ pending when finishing MQ."""
-        d = defer.Deferred()
-        self.qm.mq.queue(d)
-        self.qm.mq.queue(defer.succeed(True))
-        d.callback(True)
-
-        self.assertEqual(self.aq.execs, ['MQ', 'MQ'])
-        self.assertEqual(self.qm.state, QueueManager.IDLE)
-
-    def test_workingCQ_still_both_cm(self):
-        """Has both pending (C-M) when finishing CQ."""
-        dc = defer.Deferred()
-        dm = defer.Deferred()
-        self.qm.cq.queue(dc)
-        self.qm.cq.queue(defer.succeed(True))
-        self.qm.mq.queue(dm)
-        dc.callback(True)
-        dm.callback(True)
-
-        self.assertEqual(self.aq.execs, ['CQ', 'CQ', 'MQ'])
-        self.assertEqual(self.qm.state, QueueManager.IDLE)
-
-    def test_workingCQ_still_both_mc(self):
-        """Has both pending (M-C) when finishing CQ."""
-        dc = defer.Deferred()
-        dm = defer.Deferred()
-        self.qm.cq.queue(dc)
-        self.qm.mq.queue(dm)
-        self.qm.cq.queue(defer.succeed(True))
-        dm.callback(True)
-        dc.callback(True)
-
-        self.assertEqual(self.aq.execs, ['CQ', 'MQ', 'CQ'])
-        self.assertEqual(self.qm.state, QueueManager.IDLE)
-
-    def test_workingMQ_still_both_mc(self):
-        """Has both pending (M-C) when finishing MQ."""
-        dc = defer.Deferred()
-        dm = defer.Deferred()
-        self.qm.mq.queue(dm)
-        self.qm.mq.queue(defer.succeed(True))
-        self.qm.cq.queue(dc)
-        dm.callback(True)
-        dc.callback(True)
-
-        self.assertEqual(self.aq.execs, ['MQ', 'MQ', 'CQ'])
-        self.assertEqual(self.qm.state, QueueManager.IDLE)
-
-    def test_workingMQ_still_both_cm(self):
-        """Has both pending (C-M) when finishing MQ."""
-        # this is the assymetric one that gives MQ higher priority!
-        dc = defer.Deferred()
-        dm = defer.Deferred()
-        self.qm.mq.queue(dm)
-        self.qm.cq.queue(defer.succeed(True))
-        self.qm.mq.queue(dc)
-        dm.callback(True)
-        dc.callback(True)
-
-        self.assertEqual(self.aq.execs, ['MQ', 'MQ', 'CQ'])
-        self.assertEqual(self.qm.state, QueueManager.IDLE)
-
+        self.assertEqual(self.qm.state, QueueManager.WORKING)
+        self.assertTrue(self.aq.queue.active)
 
 
 class TestConnectionManager(Base):
@@ -627,7 +338,6 @@ class TestConnectionManager(Base):
         self.assertTrue(new_node is None) # if working, it should return a node
 
 
-
 class TestConnectionManagerTimings(Base):
     """Times handled by ConnectionManager."""
 
@@ -675,7 +385,6 @@ class TestConnectionManagerTimings(Base):
             self.assertEqual(self.sm.connection.waiting_timeout, t)
 
 
-
 class StateManagerTransitions(Base):
     """Base class for all transition tests."""
 
@@ -702,7 +411,6 @@ class StateManagerTransitions(Base):
             return self.wait_event('SYS_STATE_CHANGED')
 
 
-
 class TestStateManagerHighLevelTransitions(StateManagerTransitions):
     """Test StateManager high level transitions."""
 
@@ -799,8 +507,7 @@ class TestStateManagerHighLevelTransitions(StateManagerTransitions):
         """Don't make transition, and don't log warning."""
         nodes = ['INIT', 'READY', 'STANDOFF', 'QUEUE_MANAGER'] # examples
         d = []
-        for event in ('SYS_META_QUEUE_WAITING', 'SYS_META_QUEUE_DONE',
-                     'SYS_CONTENT_QUEUE_WAITING', 'SYS_CONTENT_QUEUE_DONE'):
+        for event in ('SYS_QUEUE_WAITING', 'SYS_QUEUE_DONE'):
             node = nodes.pop()
             d.append(self.check(node, event, node))
         return defer.DeferredList(d)
@@ -815,38 +522,13 @@ class TestStateManagerQueueTransitions(StateManagerTransitions):
         self.sm.handle_default(event)
         return self.wait_event('SYS_STATE_CHANGED')
 
-    def test_IDLE_SYS_META_QUEUE_WAITING(self):
-        """Transition from IDLE when SYS_META_QUEUE_WAITING."""
-        return self.check('IDLE', 'SYS_META_QUEUE_WAITING')
-
-    def test_IDLE_SYS_CONTENT_QUEUE_WAITING(self):
-        """Transition from IDLE when SYS_CONTENT_QUEUE_WAITING."""
-        return self.check('IDLE', 'SYS_CONTENT_QUEUE_WAITING')
-
-    def test_WORKING_ON_METADATA_SYS_META_QUEUE_DONE(self):
-        """Transition from WORKING_ON_METADATA when SYS_META_QUEUE_DONE."""
-        return self.check('WORKING_ON_METADATA', 'SYS_META_QUEUE_DONE')
-
-    def test_WORKING_ON_METADATA_SYS_CONTENT_QUEUE_WAITING(self):
-        """Transition from WORKING_ON_METADATA when CONTENT_QUEUE_WAITING."""
-        return self.check('WORKING_ON_METADATA', 'SYS_CONTENT_QUEUE_WAITING')
-
-    def test_WORKING_ON_CONTENT_SYS_META_QUEUE_WAITING(self):
-        """Transition from WORKING_ON_CONTENT when SYS_META_QUEUE_WAITING."""
-        return self.check('WORKING_ON_CONTENT', 'SYS_META_QUEUE_WAITING')
-
-    def test_WORKING_ON_CONTENT_SYS_CONTENT_QUEUE_DONE(self):
-        """Transition from WORKING_ON_CONTENT when SYS_CONTENT_QUEUE_DONE."""
-        return self.check('WORKING_ON_CONTENT', 'SYS_CONTENT_QUEUE_DONE')
-
-    def test_WORKING_ON_BOTH_SYS_META_QUEUE_DONE(self):
-        """Transition from WORKING_ON_BOTH when SYS_META_QUEUE_DONE."""
-        return self.check('WORKING_ON_BOTH', 'SYS_META_QUEUE_DONE')
-
-    def test_WORKING_ON_BOTH_SYS_CONTENT_QUEUE_DONE(self):
-        """Transition from WORKING_ON_BOTH when SYS_CONTENT_QUEUE_DONE."""
-        return self.check('WORKING_ON_BOTH', 'SYS_CONTENT_QUEUE_DONE')
+    def test_IDLE_SYS_QUEUE_WAITING(self):
+        """Transition from IDLE when SYS_QUEUE_WAITING."""
+        return self.check('IDLE', 'SYS_QUEUE_WAITING')
 
+    def test_WORKING_SYS_QUEUE_DONE(self):
+        """Transition from WORKING when SYS_QUEUE_DONE."""
+        return self.check('WORKING', 'SYS_QUEUE_DONE')
 
 
 class TestStateManagerNetworkTransitions(StateManagerTransitions):
@@ -911,7 +593,6 @@ class TestStateManagerNetworkTransitions(StateManagerTransitions):
         return defer.DeferredList(d)
 
 
-
 class TestStateManagerTimeoutTransitions(StateManagerTransitions):
     """Test StateManager when HandshakeTimeouts."""
 
@@ -945,6 +626,16 @@ class TestStateManagerTimeoutTransitions(StateManagerTransitions):
                           'STANDOFF', True)
 
 
+class TestStateManagerShutdown(StateManagerTransitions):
+    """Test StateManager when shutting down."""
+
+    def test_shutdown(self):
+        """All nodes go to SHUTDOWN."""
+        d = []
+        for node in self.sm_allnodes:
+            d.append(self.check(node, 'SYS_QUIT', 'SHUTDOWN'))
+        return defer.DeferredList(d)
+
 
 class TestStateManagerErrors(StateManagerTransitions):
     """Test StateManager on error conditions."""
@@ -974,7 +665,6 @@ class TestStateManagerErrors(StateManagerTransitions):
         return defer.DeferredList(d)
 
 
-
 class TestStateManagerEnterExit(Base):
     """Test StateManager on enter and exit transitions."""
 
@@ -1144,12 +834,10 @@ class TestStateManagerEnterExit(Base):
         events = ('SYS_NET_DISCONNECTED', 'SYS_USER_DISCONNECT',
                   'SYS_CONNECTION_LOST')
         for event in events:
+            self.aq.queue.active = True
             self.sm.state = StateManager.QUEUE_MANAGER
             self.sm.handle_default(event)
-        # we just count the cleanups because there're some on_enter mixed
-        # from STANDOFF
-        self.assertEqual(self.aq.actions.count('cleanup'), 3)
-
+            self.assertFalse(self.aq.queue.active)
 
 
 class TestStateManagerPassToNetworkManager(Base):
@@ -1227,21 +915,12 @@ class TestStateManagerPassToQueueManager(Base):
             self.assertEqual(self.called_events, [event]*cnt)
 
     def test_meta_waiting(self):
-        """SYS_META_QUEUE_WAITING should go to QueueMgr no matter where."""
-        self._test('SYS_META_QUEUE_WAITING')
+        """SYS_QUEUE_WAITING should go to QueueMgr no matter where."""
+        self._test('SYS_QUEUE_WAITING')
 
     def test_meta_done(self):
-        """SYS_META_QUEUE_DONE should go to QueueMgr no matter where."""
-        self._test('SYS_META_QUEUE_DONE')
-
-    def test_content_waiting(self):
-        """SYS_CONTENT_QUEUE_WAITING should go to QueueMgr no matter where."""
-        self._test('SYS_CONTENT_QUEUE_WAITING')
-
-    def test_content_done(self):
-        """SYS_CONTENT_QUEUE_DONE should go to QueueMgr no matter where."""
-        self._test('SYS_CONTENT_QUEUE_DONE')
-
+        """SYS_QUEUE_DONE should go to QueueMgr no matter where."""
+        self._test('SYS_QUEUE_DONE')
 
 
 class TestStateManagerAPI(Base):
@@ -1331,3 +1010,7 @@ class TestStateManagerAPI(Base):
         self.sm.state = StateManager.STANDOFF
         self.check_node("STANDOFF", error=False, conn=True, online=False)
 
+    def test_SHUTDOWN(self):
+        """SHUTDOWN internals."""
+        self.sm.state = StateManager.SHUTDOWN
+        self.check_node("SHUTDOWN", error=False, conn=False, online=False)
