@@ -20,6 +20,7 @@
 
 from __future__ import with_statement
 
+import contextlib
 import copy
 from inspect import getmembers, getargspec, ismethod
 import logging
@@ -35,12 +36,12 @@ from contrib.testing.testcase import (
     FakeMain,
     FakeVolumeManager,
     BaseTwistedTestCase,
-    MementoHandler,
     Listener,
 )
 
-from ubuntuone.syncdaemon.dbus_interface import DBusInterface
+from ubuntuone.devtools.handlers import MementoHandler
 from ubuntuone.syncdaemon.filesystem_manager import FileSystemManager
+from ubuntuone.syncdaemon.tritcask import Tritcask
 from ubuntuone.syncdaemon.fsm import fsm as fsm_module
 from ubuntuone.syncdaemon.sync import FSKey, Sync, SyncStateMachineRunner
 from ubuntuone.syncdaemon.volume_manager import Share
@@ -49,34 +50,32 @@ from ubuntuone.storageprotocol.request import ROOT
 from ubuntuone.storageprotocol import delta
 from ubuntuone.syncdaemon.marker import MDMarker
 
-DBusInterface.test = True
-
 
 class TestSyncClassAPI(unittest.TestCase):
     """Tests that make sure the Sync class has the correct API."""
 
-    def test_error_handle_signatures(self):
-        """Make sure signatures match for the error handlers.
+    def test_handle_signatures(self):
+        """Make sure signatures match for the handlers.
 
         Each handle_<EVENT> should map to an EVENT in event_queue and have the
         same signature.
         """
-        for k in [k for k in dict(getmembers(Sync)) if k.startswith('handle_')]:
-            handler = getattr(Sync, k, None)
+        handls = (k for k in dict(getmembers(Sync)) if k.startswith('handle_'))
+        for handle in handls:
+            handler = getattr(Sync, handle, None)
             # it must be a method
             self.assertTrue(ismethod(handler))
-            event = EVENTS.get(k.replace('handle_', ''))
-            # there must be a matching EVENT
-            self.assertNotEqual(event, None)
-            spec = getargspec(handler).args[1::]
+            event = EVENTS[handle.replace('handle_', '')]
+            spec = getargspec(handler).args[1:]
             # the argspec must also match (same order same variable names)
-            self.assertEqual(list(event),spec,
-                "Handler %s args do not match event args %s" % (k, event))
+            self.assertEqual(list(event), spec,
+                "Handler %s args do not match event args %s" % (handle, event))
 
 
 class FSKeyTestCase(BaseTwistedTestCase):
     """ Base test case for FSKey """
 
+    @defer.inlineCallbacks
     def setUp(self):
         """ Setup the test """
         unittest.TestCase.setUp(self)
@@ -88,29 +87,31 @@ class FSKeyTestCase(BaseTwistedTestCase):
         self.fsmdir = os.path.join(self.test_dir, "fsmdir")
         self.partials_dir = os.path.join(self.test_dir, "partials")
         os.makedirs(self.partials_dir)
+        self.tritcask_dir = self.mktemp("tritcask_dir")
+        self.db = Tritcask(self.tritcask_dir)
         self.fsm = FileSystemManager(self.fsmdir, self.partials_dir,
-                                     FakeVolumeManager(self.root_dir))
+                                     FakeVolumeManager(self.root_dir),
+                                     self.db)
         self.eq = EventQueue(self.fsm)
         self.fsm.register_eq(self.eq)
-        self.share = self.create_share('share', 'share_name',
-                                            self.fsm, self.shares_dir)
+        self.share = yield self.create_share('share', 'share_name')
         self.share_path = self.share.path
 
     def tearDown(self):
         """ Clean up the test dir"""
         self.eq.shutdown()
+        self.db.shutdown()
         shutil.rmtree(self.test_dir)
 
-    @staticmethod
-    def create_share(share_id, share_name, fsm, shares_dir,
-                     access_level='Modify'):
-        """ creates a share """
-        share_path = os.path.join(shares_dir, share_name)
+    @defer.inlineCallbacks
+    def create_share(self, share_id, share_name, access_level='Modify'):
+        """Create a share."""
+        share_path = os.path.join(self.shares_dir, share_name)
         os.makedirs(share_path)
         share = Share(path=share_path, volume_id=share_id,
                       access_level=access_level)
-        fsm.vm.add_share(share)
-        return share
+        yield self.fsm.vm.add_share(share)
+        defer.returnValue(share)
 
 
 class FSKeyTests(FSKeyTestCase):
@@ -254,6 +255,7 @@ class TestSync(BaseSync):
         BaseSync.setUp(self)
         self.sync = Sync(main=self.main)
         self.fsm = self.main.fs
+        self.handler.setLevel(logging.DEBUG)
 
     def test_deleting_open_files_is_no_cause_for_despair(self):
         """test_deleting_open_files_is_no_cause_for_despair."""
@@ -339,6 +341,21 @@ class TestSync(BaseSync):
         self.sync._handle_SV_HASH_NEW(mdobj.share_id, mdobj.node_id, '')
         self.assertTrue(self.called)
 
+    def test_SV_HASH_NEW_with_file_uploadinterrupted_coverage(self):
+        """A SV_HASH_NEW is received after upload interrupted."""
+        self.called = False
+
+        # create a file and put it in local, without server_hash, as
+        # if the upload was cut in the middle after the make file
+        somepath = os.path.join(self.root, 'somepath')
+        mdid = self.fsm.create(somepath, '', node_id='node_id')
+        self.fsm.set_by_mdid(mdid, local_hash='somehash', crc32='crc32',
+                             stat='stat', size='size')
+
+        # send the event with no content and check
+        mdobj = self.fsm.get_by_mdid(mdid)
+        self.sync._handle_SV_HASH_NEW(mdobj.share_id, mdobj.node_id, '')
+
     def test_AQ_FILE_NEW_OK_with_md_in_none(self):
         """Created the file, and MD says it's in NONE."""
         # fake method
@@ -605,6 +622,76 @@ class TestSync(BaseSync):
                                        'new_parent_id', 'new_name', 'error')
         self.assertEqual(called[1:], ['AQ_MOVE_ERROR', {}, '', 'node_id'])
 
+    def test_SV_FILE_NEW_no_node(self):
+        """Handle SV_FILE_NEW not having a node."""
+        # fake method
+        called = []
+        orig = SyncStateMachineRunner.new_file
+        self.patch(SyncStateMachineRunner, 'new_file',
+                   lambda *a: called.extend(a) or orig(*a))
+
+        # create the parent
+        parentpath = os.path.join(self.root, 'somepath')
+        self.fsm.create(parentpath, '', node_id='parent_id')
+
+        # call and check
+        r = self.sync._handle_SV_FILE_NEW('', 'node_id', 'parent_id', 'name')
+        self.assertEqual(called[1:], ['SV_FILE_NEW', {}, '', 'node_id',
+                                      'parent_id', 'name'])
+        self.assertEqual(r.share_id, '')
+        self.assertEqual(r.node_id, 'node_id')
+        self.assertEqual(r.path, 'somepath/name')
+
+    def test_SV_FILE_NEW_node_no_id(self):
+        """Handle SV_FILE_NEW having a node without node_id."""
+        # fake method
+        called = []
+        orig = SyncStateMachineRunner.new_server_file_having_local
+        self.patch(SyncStateMachineRunner, 'new_server_file_having_local',
+                   lambda *a: called.extend(a) or orig(*a))
+
+        # create the node and its parent, to match per path
+        parentpath = os.path.join(self.root, 'somepath')
+        self.fsm.create(parentpath, '', node_id='parent_id')
+        childpath = os.path.join(parentpath, 'name')
+        self.fsm.create(childpath, '')
+
+        # call and check
+        self.sync._handle_SV_FILE_NEW('', 'node_id', 'parent_id', 'name')
+        self.assertEqual(called[1:], ['SV_FILE_NEW', {}, '', 'node_id',
+                                      'parent_id', 'name'])
+
+    def test_SV_FILE_NEW_node_same_id(self):
+        """Handle SV_FILE_NEW having a node with the same node_id."""
+        parentpath = os.path.join(self.root, 'somepath')
+        self.fsm.create(parentpath, '', node_id='parent_id')
+        childpath = os.path.join(parentpath, 'name')
+        self.fsm.create(childpath, '', node_id='node_id')
+        r = self.assertRaises(ValueError, self.sync._handle_SV_FILE_NEW,
+                              '', 'node_id', 'parent_id', 'name')
+        self.assertTrue("same node_id in handle_SV_FILE_NEW" in str(r))
+
+    def test_SV_FILE_NEW_node_different_id(self):
+        """Handle SV_FILE_NEW having a node with different node_id."""
+        # create the node and its parent, to match per path
+        parentpath = os.path.join(self.root, 'somepath')
+        self.fsm.create(parentpath, '', node_id='parent_id')
+        childpath = os.path.join(parentpath, 'name')
+        self.fsm.create(childpath, '', node_id='other_id')
+
+        # fake method
+        called = []
+        orig = self.fsm.delete_file
+        self.patch(self.fsm, 'delete_file',
+                   lambda path: called.append(path) or orig(path))
+
+        # call and check
+        self.sync._handle_SV_FILE_NEW('', 'node_id', 'parent_id', 'name')
+        self.assertEqual(called, [childpath])
+        self.handler.debug  = True
+        self.assertTrue(self.handler.check_debug("Wanted to apply SV_FILE_NEW",
+                                                 "found it with other id"))
+
 
 class SyncStateMachineRunnerTestCase(BaseSync):
     """Tests for the SyncStateMachineRunner."""
@@ -628,7 +715,6 @@ class SyncStateMachineRunnerTestCase(BaseSync):
         self.handler.setLevel(logging.DEBUG)
         self.ssmr.log.addHandler(self.handler)
 
-
     def tearDown(self):
         """Clean up."""
         self.ssmr.log.removeHandler(self.handler)
@@ -683,6 +769,55 @@ class SyncStateMachineRunnerTestCase(BaseSync):
         self.assertEqual(mdobj.size, 'size')
         self.assertEqual(mdobj.stat, 'stt')
 
+    @contextlib.contextmanager
+    def _test_putcontent_upload_id(self, with_upload_id=False):
+        """Generic code to setup and check upload_id in put_content."""
+        if with_upload_id:
+            self.fsm.set_by_mdid(self.mdid, upload_id='hola')
+        else:
+            self.fsm.set_by_mdid(self.mdid, upload_id=None)
+        called = []
+        def my_upload(*args, **kwargs):
+            """AQ.upload method that only collect the arguments."""
+            called.append((args, kwargs))
+        self.patch(self.aq, 'upload', my_upload)
+        yield
+        self.assertEqual(len(called), 1)
+        kwargs = called[0][1]
+        if with_upload_id:
+            self.assertEqual(kwargs['upload_id'], 'hola')
+        else:
+            self.assertEqual(kwargs['upload_id'], None)
+
+    def test_put_file_use_upload_id(self):
+        """Test that sync calls put_file with the correct args."""
+        with self._test_putcontent_upload_id():
+            self.ssmr.put_file('HQ_HASH_NEW', None, 'hash',
+                               'crc', 'size', 'stt')
+        with self._test_putcontent_upload_id(with_upload_id=False):
+            self.ssmr.put_file('HQ_HASH_NEW', None, 'hash',
+                               'crc', 'size', 'stt')
+
+    def test_reput_file_use_upload_id(self):
+        """Test that sync calls reput_file with the correct args."""
+        with self._test_putcontent_upload_id():
+            self.ssmr.reput_file('HQ_HASH_NEW', None, 'hash',
+                                 'crc', 'size', 'stt')
+        with self._test_putcontent_upload_id(with_upload_id=False):
+            self.ssmr.reput_file('HQ_HASH_NEW', None, 'hash',
+                                 'crc', 'size', 'stt')
+
+    def test_reput_file_from_local_use_upload_id(self):
+        """Test that sync calls reput_file_from_local with the correct args."""
+        self.fsm.set_by_mdid(self.mdid, local_hash='somehash', crc32='crc32',
+                             stat='stat', size='size')
+        # send the event with no content and check
+        with self._test_putcontent_upload_id():
+            self.ssmr.reput_file_from_local("SV_HASH_NEW", None, '')
+        with self._test_putcontent_upload_id(with_upload_id=False):
+            self.ssmr.reput_file_from_local("SV_HASH_NEW", None, '')
+
+
     def test_commit_file_without_partial(self):
         """The .partial is lost when commiting the file."""
         # create the partial correctly, and break it!
@@ -760,6 +895,182 @@ class SyncStateMachineRunnerTestCase(BaseSync):
         self.assertEqual(result, 'new_id')
         self.assertEqual(called, [('marker', 'new_id')])
 
+    def test_file_delete_on_server_sends_is_dir(self):
+        """delete_on_server sends the is_dir flag."""
+        somepath = os.path.join(self.root, 'foo')
+        mdid = self.fsm.create(somepath, '', is_dir=False)
+
+        # patch to control the call to key
+        called = []
+
+        # create context and call
+        key = FSKey(self.main.fs, path=somepath)
+        ssmr = SyncStateMachineRunner(fsm=self.fsm, main=self.main,
+                                      key=key, logger=None)
+        self.patch(self.main.action_q, "unlink",
+                   lambda *args: called.append(args))
+
+        ssmr.delete_on_server(None, None, somepath)
+
+        # check
+        self.assertEqual(called[0][-3:], (mdid, somepath, False))
+
+    def test_folder_delete_on_server_sends_is_dir(self):
+        """delete_on_server sends the is_dir flag."""
+        somepath = os.path.join(self.root, 'foo')
+        mdid = self.fsm.create(somepath, '', is_dir=True)
+
+        # patch to control the call to key
+        called = []
+
+        # create context and call
+        key = FSKey(self.main.fs, path=somepath)
+        ssmr = SyncStateMachineRunner(fsm=self.fsm, main=self.main,
+                                      key=key, logger=None)
+        self.patch(self.main.action_q, "unlink",
+                   lambda *args: called.append(args))
+
+        ssmr.delete_on_server(None, None, somepath)
+
+        # check
+        self.assertEqual(called[0][-3:], (mdid, somepath, True))
+
+    def test_file_deleted_dir_while_downloading_sends_is_dir(self):
+        """Deleted parent while file is downloading sends the is_dir flag."""
+        somepath = os.path.join(self.root, 'foo')
+        mdid = self.fsm.create(somepath, '', is_dir=False)
+
+        # patch to control the call to key
+        called = []
+
+        # create context and call
+        self.patch(FSKey, "remove_partial", lambda o: None)
+        key = FSKey(self.main.fs, path=somepath)
+        ssmr = SyncStateMachineRunner(fsm=self.fsm, main=self.main,
+                                      key=key, logger=None)
+        self.patch(self.main.action_q, "cancel_download",
+                   lambda share_id, node_id: None)
+        self.patch(self.main.action_q, "unlink",
+                   lambda *args: called.append(args))
+
+        ssmr.deleted_dir_while_downloading(None, None, somepath)
+
+        # check
+        self.assertEqual(called[0][-3:], (mdid, somepath, False))
+
+    def test_folder_deleted_dir_while_downloading_sends_is_dir(self):
+        """Deleted parent while dir is downloading sends the is_dir flag."""
+        somepath = os.path.join(self.root, 'foo')
+        mdid = self.fsm.create(somepath, '', is_dir=True)
+
+        # patch to control the call to key
+        called = []
+
+        # create context and call
+        self.patch(FSKey, "remove_partial", lambda o: None)
+        key = FSKey(self.main.fs, path=somepath)
+        ssmr = SyncStateMachineRunner(fsm=self.fsm, main=self.main,
+                                      key=key, logger=None)
+        self.patch(self.main.action_q, "cancel_download",
+                   lambda share_id, node_id: None)
+        self.patch(self.main.action_q, "unlink",
+                   lambda *args: called.append(args))
+
+        ssmr.deleted_dir_while_downloading(None, None, somepath)
+
+        # check
+        self.assertEqual(called[0][-3:], (mdid, somepath, True))
+
+    def test_file_cancel_download_and_delete_on_server_sends_is_dir(self):
+        """cancel_download_and_delete_on_server sends the is_dir flag."""
+        somepath = os.path.join(self.root, 'foo')
+        mdid = self.fsm.create(somepath, '', is_dir=False)
+
+        # patch to control the call to key
+        called = []
+
+        # create context and call
+        self.patch(FSKey, "remove_partial", lambda o: None)
+        key = FSKey(self.main.fs, path=somepath)
+        ssmr = SyncStateMachineRunner(fsm=self.fsm, main=self.main,
+                                      key=key, logger=None)
+        self.patch(self.main.action_q, "cancel_download",
+                   lambda share_id, node_id: None)
+        self.patch(self.main.action_q, "unlink",
+                   lambda *args: called.append(args))
+
+        ssmr.cancel_download_and_delete_on_server(None, None, somepath)
+
+        # check
+        self.assertEqual(called[0][-3:], (mdid, somepath, False))
+
+    def test_folder_cancel_download_and_delete_on_server_sends_is_dir(self):
+        """cancel_download_and_delete_on_server sends the is_dir flag."""
+        somepath = os.path.join(self.root, 'foo')
+        mdid = self.fsm.create(somepath, '', is_dir=True)
+
+        # patch to control the call to key
+        called = []
+
+        # create context and call
+        self.patch(FSKey, "remove_partial", lambda o: None)
+        key = FSKey(self.main.fs, path=somepath)
+        ssmr = SyncStateMachineRunner(fsm=self.fsm, main=self.main,
+                                      key=key, logger=None)
+        self.patch(self.main.action_q, "cancel_download",
+                   lambda share_id, node_id: None)
+        self.patch(self.main.action_q, "unlink",
+                   lambda *args: called.append(args))
+
+        ssmr.cancel_download_and_delete_on_server(None, None, somepath)
+
+        # check
+        self.assertEqual(called[0][-3:], (mdid, somepath, True))
+
+    def test_file_cancel_upload_and_delete_on_server_sends_is_dir(self):
+        """cancel_upload_and_delete_on_server sends the is_dir flag."""
+        somepath = os.path.join(self.root, 'foo')
+        mdid = self.fsm.create(somepath, '', is_dir=False)
+
+        # patch to control the call to key
+        called = []
+
+        # create context and call
+        key = FSKey(self.main.fs, path=somepath)
+        ssmr = SyncStateMachineRunner(fsm=self.fsm, main=self.main,
+                                      key=key, logger=None)
+        self.patch(self.main.action_q, "cancel_download",
+                   lambda share_id, node_id: None)
+        self.patch(self.main.action_q, "unlink",
+                   lambda *args: called.append(args))
+
+        ssmr.cancel_upload_and_delete_on_server(None, None, somepath)
+
+        # check
+        self.assertEqual(called[0][-3:], (mdid, somepath, False))
+
+    def test_folder_cancel_upload_and_delete_on_server_sends_is_dir(self):
+        """cancel_upload_and_delete_on_server sends the is_dir flag."""
+        somepath = os.path.join(self.root, 'foo')
+        mdid = self.fsm.create(somepath, '', is_dir=True)
+
+        # patch to control the call to key
+        called = []
+
+        # create context and call
+        key = FSKey(self.main.fs, path=somepath)
+        ssmr = SyncStateMachineRunner(fsm=self.fsm, main=self.main,
+                                      key=key, logger=None)
+        self.patch(self.main.action_q, "cancel_download",
+                   lambda share_id, node_id: None)
+        self.patch(self.main.action_q, "unlink",
+                   lambda *args: called.append(args))
+
+        ssmr.cancel_upload_and_delete_on_server(None, None, somepath)
+
+        # check
+        self.assertEqual(called[0][-3:], (mdid, somepath, True))
+
     @defer.inlineCallbacks
     def test_filedir_error_in_creation(self):
         """Conflict and delete metada, and release the marker with error."""
@@ -837,8 +1148,10 @@ class SyncStateMachineRunnerTestCase(BaseSync):
         key = FSKey(self.main.fs, path=somepath1)
         ssmr = SyncStateMachineRunner(fsm=self.fsm, main=self.main,
                                       key=key, logger=None)
+        parent_id = FSKey(self.main.fs, path=self.root)['node_id']
         ssmr.client_moved('some event', {}, somepath1, somepath2)
-        self.assertTrue(called)
+        self.assertEqual(called, [('', 'node_id', parent_id, parent_id,
+                                   'bar', somepath1, somepath2)])
 
     def test_clean_move_limbo(self):
         """Clean the move limbo with what was called."""
@@ -873,6 +1186,24 @@ class SyncStateMachineRunnerTestCase(BaseSync):
         # check the create and the make_dir
         self.assertEqual(called, [(new_path, '', 'node_id', True), 'mdid'])
 
+    def test_new_new_server_file_having_local(self):
+        """Set the node_id to the node."""
+        # create context
+        somepath = os.path.join(self.root, 'foo')
+        self.fsm.create(somepath, '')
+        key = FSKey(self.main.fs, path=somepath)
+        ssmr = SyncStateMachineRunner(fsm=self.fsm, main=self.main,
+                                      key=key, logger=None)
+
+        # log the call to fsm
+        called = []
+        self.fsm.set_node_id = lambda *a: called.append(a)
+
+        # call the tested method and check
+        ssmr.new_server_file_having_local('event', {}, '', 'node_id',
+                                          'parent_id', 'name')
+        self.assertEqual(called, [(somepath, 'node_id')])
+
 
 class FakedState(object):
     """A faked state."""
@@ -946,7 +1277,7 @@ class TestNewGenerationOnOperations(BaseSync):
                    lambda s, *a: called.append(a))
 
         d = dict(share_id='volume_id', node_id='node_id', parent_id='parent',
-                 new_generation=77)
+                 new_generation=77, was_dir=False, old_path="test path")
         self.sync.handle_AQ_UNLINK_OK(**d)
         self.assertEqual(called, [('volume_id', "node_id", 77)])
 
@@ -1453,7 +1784,7 @@ class TestHandleAqDeltaOk(TestSyncDelta):
         # check event
         kwargs = dict(volume_id=mdobj.share_id, node_id=mdobj.node_id,
                       path=mdobj.path, mdid=mdobj.mdid)
-        self.assertTrue(("SYS_BROKEN_NODE", (), kwargs) in listener.events)
+        self.assertTrue(("SYS_BROKEN_NODE", kwargs) in listener.events)
 
     def test_dirty_node_sends_event_nonode(self):
         """When a node is marked dirty, an event should fly, no node."""
@@ -1465,7 +1796,7 @@ class TestHandleAqDeltaOk(TestSyncDelta):
 
         # check event
         kwargs = dict(volume_id='volume', node_id='node', path=None, mdid=None)
-        self.assertTrue(("SYS_BROKEN_NODE", (), kwargs) in listener.events)
+        self.assertTrue(("SYS_BROKEN_NODE", kwargs) in listener.events)
 
     def test_dirty_node_logs_special(self):
         """When a node is marked dirty, it should log in a special handler."""
@@ -1521,7 +1852,8 @@ class TestHandleAqRescanFromScratchOk(TestSyncDelta):
         self.sync.handle_AQ_RESCAN_FROM_SCRATCH_OK(ROOT, [self.rootdt],
                                                    100, 100)
 
-        self.assertEqual(called, [((ROOT, self.filetxtdelta.node_id), {})])
+        args = (ROOT, self.filetxtdelta.node_id, False)
+        self.assertEqual(called, [(args, {})])
 
     def test_deletes_file_in_delta(self):
         """Files in delta should not be deleted."""
@@ -1572,5 +1904,72 @@ class TestHandleAqRescanFromScratchOk(TestSyncDelta):
             [self.rootdt], 100, 100)
 
         self.assertEqual(called, [
-            (ROOT, dt.node_id),
-            (ROOT, self.dirdelta.node_id)])
+            (ROOT, dt.node_id, False),
+            (ROOT, self.dirdelta.node_id, True)])
+
+
+class TestSyncEvents(TestSyncDelta):
+    """Testing sync stuff related to events."""
+
+    def setUp(self):
+        """Do the setUp."""
+        super(TestSyncEvents, self).setUp()
+        self.sync = Sync(main=self.main)
+        self.handler.setLevel(logging.DEBUG)
+
+        key = FSKey(self.main.fs, share_id='', node_id='node_id')
+        self.ssmr = SyncStateMachineRunner(fsm=self.main.fs, main=self.main,
+                                           key=key, logger=None)
+        self.vm = self.main.vm
+        self.listener = Listener()
+        self.main.event_q.subscribe(self.listener)
+
+    def test_server_new_file_sends_event(self):
+        """When a new file is created on the server, an event is sent."""
+        # create the fake file
+        parent_id = self.root_id
+        self.sync._handle_SV_FILE_NEW(ROOT, "node_id", parent_id, "file")
+
+        # check event
+        kwargs = dict(volume_id=ROOT, node_id='node_id', parent_id=parent_id,
+                      name="file")
+        self.assertIn(("SV_FILE_NEW", kwargs), self.listener.events)
+
+    def test_server_new_dir_sends_event(self):
+        """When a new directory is created on the server, an event is sent."""
+        # create the fake dir
+        parent_id = self.root_id
+        self.sync._handle_SV_DIR_NEW(ROOT, "node_id", parent_id, "file")
+
+        # check event
+        kwargs = dict(volume_id=ROOT, node_id='node_id', parent_id=parent_id,
+                      name="file")
+        self.assertIn(("SV_DIR_NEW", kwargs), self.listener.events)
+
+    def test_server_file_deleted_sends_event(self):
+        """When a file is deleted, an event is sent."""
+        node = self.create_filetxt()
+        full_path = self.main.fs.get_abspath(node.share_id, node.path)
+
+        # delete the fake file
+        self.sync._handle_SV_FILE_DELETED(ROOT, node.node_id, True)
+
+        # check event
+        kwargs = dict(volume_id=ROOT, node_id=node.node_id, was_dir=True,
+                      old_path=full_path)
+        self.assertIn(("SV_FILE_DELETED", kwargs), self.listener.events)
+
+    def test_server_file_deleted_ignores_missing_mdid(self):
+        """On file delete, we ignore missing metadata."""
+        node = self.create_filetxt()
+
+        # delete the fake file *twice*
+        self.sync._handle_SV_FILE_DELETED(ROOT, node.node_id, True)
+        self.listener.events = []
+        self.sync._handle_SV_FILE_DELETED(ROOT, node.node_id, True)
+
+        # the old_path is not available on the second call
+        kwargs = dict(volume_id=ROOT, node_id=node.node_id, was_dir=True,
+                      old_path="")
+        # check event
+        self.assertIn(("SV_FILE_DELETED", kwargs), self.listener.events)
