@@ -57,6 +57,7 @@ from ubuntuone.storageprotocol.client import (
 from ubuntuone.storageprotocol.context import get_ssl_context
 from ubuntuone.syncdaemon.interfaces import IActionQueue, IMarker
 from ubuntuone.syncdaemon.logger import mklog, TRACE
+from ubuntuone.syncdaemon import config
 
 logger = logging.getLogger("ubuntuone.SyncDaemon.ActionQueue")
 
@@ -69,8 +70,6 @@ EREGEX = "^.+\\@(\\[?)[a-zA-Z0-9\\-\\.]+\\.([a-zA-Z]{2,3}|[0-9]{1,3})(\\]?)$"
 # progress threshold to emit a download/upload progress event: 64Kb
 TRANSFER_PROGRESS_THRESHOLD = 64*1024*1024
 
-# no more than this uploads are executed simultaneously
-SIMULT_TRANSFERS = 3
 
 def passit(func):
     """Pass the value on for the next deferred, while calling func with it."""
@@ -302,6 +301,11 @@ class ActionQueueProtocol(LoggingStorageClient):
     _looping_ping = None
     _ping_delay = 600  # 10 minutes
 
+    def __init__(self):
+        LoggingStorageClient.__init__(self)
+        user_config = config.get_user_config()
+        self.max_payload_size = user_config.get_max_payload_size()
+
     def connectionMade(self):
         """A new connection was made."""
         self.log.info('Connection made.')
@@ -458,9 +462,13 @@ class RequestQueue(object):
         self.waiting = []
         self.hashed_waiting = {}
         self.active = False
-        self.transfers_semaphore = defer.DeferredSemaphore(SIMULT_TRANSFERS)
         self.active_deferred = defer.Deferred()
 
+        # transfers semaphore
+        user_config = config.get_user_config()
+        simult_transfers = user_config.get_simult_transfers()
+        self.transfers_semaphore = defer.DeferredSemaphore(simult_transfers)
+
     def __len__(self):
         """Return the length of the waiting queue."""
         return len(self.waiting)
