@@ -374,6 +374,26 @@ class _Config(SyncDaemonConfigParser):
     def set_use_trash(self, enabled):
         self.set(MAIN, 'use_trash', str(enabled))
 
+    @requires_section(MAIN)
+    def get_simult_transfers(self):
+        """Get the simultaneous transfers value."""
+        return self.get_parsed(MAIN, 'simult_transfers')
+
+    @requires_section(MAIN)
+    def set_simult_transfers(self, value):
+        """Set the simultaneous transfers value."""
+        self.set(MAIN, 'simult_transfers', str(value))
+
+    @requires_section(MAIN)
+    def get_max_payload_size(self):
+        """Get the maximum payload size."""
+        return self.get_parsed(MAIN, 'max_payload_size')
+
+    @requires_section(MAIN)
+    def set_max_payload_size(self, value):
+        """Set the maximum payload size."""
+        self.set(MAIN, 'max_payload_size', str(value))
+
     @requires_section(NOTIFICATIONS)
     def set_show_all_notifications(self, enabled):
         self.set(NOTIFICATIONS, 'show_all_notifications', str(enabled))
