@@ -22,32 +22,30 @@ import logging
 import os
 import sys
 
-import dbus
-
-from dbus.mainloop.glib import DBusGMainLoop
 from twisted.internet import defer, reactor, task
 
 from ubuntuone.syncdaemon import (
     action_queue,
     config,
-    dbus_interface,
     event_queue,
     filesystem_manager,
     hash_queue,
     events_nanny,
     local_rescan,
     sync,
+    tritcask,
     volume_manager,
 )
-from ubuntuone import syncdaemon
+from ubuntuone import syncdaemon, clientdefs
+import ubuntuone.platform
 
+from ubuntuone.platform import event_logging
+from ubuntuone.syncdaemon import status_listener
 from ubuntuone.syncdaemon.states import StateManager, QueueManager
 
 
 class WaitingHelpingHandler(object):
-    """
-    An auxiliary class that helps wait for events
-    """
+    """An auxiliary class that helps wait for events."""
     def __init__(self, event_queue, waiting_events, waiting_kwargs,
                  result=None):
         self.deferred = defer.Deferred()
@@ -57,8 +55,8 @@ class WaitingHelpingHandler(object):
         self.waiting_kwargs = waiting_kwargs
         event_queue.subscribe(self)
 
-    def handle_default(self, event, *args, **kwargs):
-        """Got an event: fire if it's one we want"""
+    def handle_default(self, event, **kwargs):
+        """Got an event: fire if it's one we want."""
         if event in self.waiting_events:
             for wk, wv in self.waiting_kwargs.items():
                 if not (wk in kwargs and kwargs[wk] == wv):
@@ -77,15 +75,17 @@ class Main(object):
     def __init__(self, root_dir, shares_dir, data_dir, partials_dir,
                  host='fs-1.one.ubuntu.com', port=443, dns_srv=None, ssl=True,
                  disable_ssl_verify=False, glib_loop=False,
-                 mark_interval=120, dbus_events=False, handshake_timeout=30,
+                 mark_interval=120, broadcast_events=False,
+                 handshake_timeout=30,
                  shares_symlink_name='Shared With Me',
                  read_limit=None, write_limit=None, throttling_enabled=False,
-                 ignore_files=None):
+                 ignore_files=None, oauth_credentials=None):
         self.root_dir = root_dir
         self.shares_dir = shares_dir
         self.shares_dir_link = os.path.join(self.root_dir, shares_symlink_name)
         self.data_dir = data_dir
         self.partials_dir = partials_dir
+        self.tritcask_dir = os.path.join(self.data_dir, 'tritcask')
         self.logger = logging.getLogger('ubuntuone.SyncDaemon.Main')
         user_config = config.get_user_config()
         if read_limit is None:
@@ -95,9 +95,12 @@ class Main(object):
         if not throttling_enabled:
             throttling_enabled = user_config.get_throttling()
 
+        self.logger.info("Starting Ubuntu One client version %s",
+                         clientdefs.VERSION)
+        self.db = tritcask.Tritcask(self.tritcask_dir)
         self.vm = volume_manager.VolumeManager(self)
-        self.fs = filesystem_manager.FileSystemManager(data_dir,
-                                                       partials_dir, self.vm)
+        self.fs = filesystem_manager.FileSystemManager(
+            data_dir, partials_dir, self.vm, self.db)
         self.event_q = event_queue.EventQueue(self.fs, ignore_files)
         self.fs.register_eq(self.event_q)
 
@@ -122,28 +125,38 @@ class Main(object):
         self.lr = local_rescan.LocalRescan(self.vm, self.fs,
                                            self.event_q, self.action_q)
 
-        if not glib_loop:
-            self.bus = dbus.SessionBus()
-        else:
-            loop = DBusGMainLoop(set_as_default=True)
-            self.bus = dbus.SessionBus(loop)
-        self.dbus_iface = dbus_interface.DBusInterface(self.bus, self,
-                                                       send_events=dbus_events)
-        self.action_q.content_queue.set_change_notification_cb(
-            self.dbus_iface.status.emit_content_queue_changed)
+        self.external = ubuntuone.platform.ExternalInterface(
+            self, glib_loop, broadcast_events)
+        self.external.oauth_credentials = oauth_credentials
+        if user_config.get_autoconnect():
+            self.external.connect(autoconnecting=True)
+
+        self.eventlog_listener = None
+        self.start_event_logger()
+        self.start_status_listener()
         self.logger.info("Using %r as root dir", self.root_dir)
         self.logger.info("Using %r as data dir", self.data_dir)
         self.logger.info("Using %r as shares root dir", self.shares_dir)
         self.mark = task.LoopingCall(self.log_mark)
         self.mark.start(mark_interval)
 
+    def start_event_logger(self):
+        """Start the event logger if it's available for this platform."""
+        self.eventlog_listener = event_logging.get_listener(self.fs, self.vm)
+        if self.eventlog_listener:
+            self.event_q.subscribe(self.eventlog_listener)
+
+    def start_status_listener(self):
+        """Start the status listener if it is configured to start."""
+        self.status_listener = status_listener.get_listener(self.fs, self.vm)
+        if self.status_listener:
+            self.event_q.subscribe(self.status_listener)
+
     def log_mark(self):
         """Log a "mark" that includes the current AQ state and queue size."""
-        self.logger.note("---- MARK (state: %s; queues: metadata: %d; content:"
-                         " %d; hash: %d, fsm-cache: hit=%d miss=%d) ----",
-                         self.state_manager, len(self.action_q.meta_queue),
-                         len(self.action_q.content_queue), len(self.hash_q),
-                         self.fs.fs.cache_hits, self.fs.fs.cache_misses)
+        self.logger.note("---- MARK (state: %s; queue: %d; hash: %d) ----",
+                         self.state_manager, len(self.action_q.queue),
+                         len(self.hash_q))
 
     def wait_for_nirvana(self, last_event_interval=0.5):
         """Get a deferred that will fire on Nirvana.
@@ -160,14 +173,12 @@ class Main(object):
             """Event queue is empty."""
             if not (self.state_manager.state == StateManager.QUEUE_MANAGER
                     and self.state_manager.queues.state == QueueManager.IDLE
-                    and not self.action_q.meta_queue
-                    and not self.action_q.content_queue
+                    and not self.action_q.queue
                     and self.hash_q.empty()):
                 self.logger.debug("I can't attain Nirvana yet. [state: %s; "
-                                  "metadata: %d; content: %d; hash: %d]",
+                                  "queue: %d; hash: %d]",
                                   self.state_manager,
-                                  len(self.action_q.meta_queue),
-                                  len(self.action_q.content_queue),
+                                  len(self.action_q.queue),
                                   len(self.hash_q))
                 return
             self.logger.debug("Nirvana reached!! I'm a Buddha")
@@ -186,21 +197,24 @@ class Main(object):
         """Setup the daemon to be ready to run."""
         # hook the event queue to the root dir
         self.event_q.push('SYS_INIT_DONE')
-        self.event_q.inotify_add_watch(self.root_dir)
+        self.event_q.add_watch(self.root_dir)
 
     def shutdown(self, with_restart=False):
         """Shutdown the daemon; optionally restart it."""
         self.event_q.push('SYS_USER_DISCONNECT')
+        self.event_q.push('SYS_QUIT')
         self.event_q.shutdown()
         self.hash_q.shutdown()
+        self.db.shutdown()
         self.state_manager.shutdown()
-        self.dbus_iface.shutdown(with_restart)
+        self.external.shutdown(with_restart)
         self.mark.stop()
 
     def restart(self):
         """Restart the daemon.
 
-        This ultimately shuts down the daemon and asks dbus to start one again.
+        This ultimately shuts down the daemon and asks external interface
+        to start one again.
         """
         self.quit(exit_value=42, with_restart=True)
 
@@ -225,6 +239,14 @@ class Main(object):
         """Return the base dir/mount point."""
         return self.root_dir
 
+    def get_sharesdir(self):
+        """Return the dir/mount point for shares."""
+        return self.shares_dir
+
+    def get_sharesdir_link(self):
+        """Return the dir link for shares."""
+        return self.shares_dir_link
+
     def quit(self, exit_value=0, with_restart=False):
         """Shutdown and stop the reactor."""
         self.shutdown(with_restart)
